#include <page/page.h>
#include <cache/slab.h>
#include <cache/cache.h>

Bool
slab_create (Slab ** result, Cache * cache)
{
    PageAddress page;
    Slab * slab;
   
    if (cache -> objectSize > SLAB_SIZE - sizeof (Slab))
      return FALSE;

    page = page_allocate (0, PAGE_ALLOCATOR_ATTRIBUTES_ANY, cache);
    if (page == 0)
      return FALSE;
      
    slab = (Slab *) (page * SLAB_SIZE);

    slab -> objectsAllocatedCount = 0;
   
    list_clear (& slab -> objectsFree);
    
    slab -> page  = page;
    slab -> cache = cache;

    list_insert_vector (list_begin (& slab -> objectsFree),
                        slab + 1,
			(uint8 *) slab + SLAB_SIZE,
			cache -> objectSize);
    
    list_insert_front (& cache -> slabsUsed, & slab -> slabList);

    cache -> slabNotEmpty = slab;

    if (result != NULL)
      * result = slab;
    
    return TRUE;
}

void
slab_destroy (Slab * slab)
{
    if (slab -> cache -> slabNotEmpty == slab)
      slab -> cache -> slabNotEmpty = NULL;

    list_remove (& slab -> slabList);

    page_free (slab -> page);
}

