#include <memory/memory.h>
#include <canvas/canvas_default.h>
#include <canvas/canvas_32.h>
#include <canvas/canvas_16.h>
#include <canvas/canvas_8.h>
#include <canvas/canvas_4.h>
#include <canvas/canvas_1.h>
#include <canvas/canvas.h>

static CanvasType canvas32Type =
{
    canvas_32_line,
    canvas_32_horizontal_line,
    canvas_32_vertical_line,
    canvas_32_points,
    canvas_32_rectangle,
    canvas_default_triangle,
    canvas_default_polygon,
    canvas_default_fill,
    canvas_32_blit,
    canvas_32_bitmap
};

static CanvasType canvas16Type =
{
    canvas_16_line,
    canvas_16_horizontal_line,
    canvas_16_vertical_line,
    canvas_16_points,
    canvas_16_rectangle,
    canvas_default_triangle,
    canvas_default_polygon,
    canvas_default_fill,
    canvas_16_blit,
    canvas_16_bitmap
};

static CanvasType canvas8Type =
{
    canvas_8_line,
    canvas_8_horizontal_line,
    canvas_8_vertical_line,
    canvas_8_points,
    canvas_8_rectangle,
    canvas_default_triangle,
    canvas_default_polygon,
    canvas_default_fill,
    canvas_8_blit,
    canvas_8_bitmap
};

static CanvasType canvas4Type = 
{
    canvas_default_line,
    canvas_4_horizontal_line,
    canvas_4_vertical_line,
    canvas_4_points,
    canvas_default_rectangle,
    canvas_default_triangle,
    canvas_default_polygon,
    canvas_default_fill,
    canvas_4_blit,
    canvas_4_bitmap
};

static CanvasType canvas1Type =
{
    canvas_default_line,
    canvas_1_horizontal_line,
    canvas_1_vertical_line,
    canvas_1_points,
    canvas_default_rectangle,
    canvas_default_triangle,
    canvas_default_polygon,
    canvas_default_fill,
    canvas_1_blit,
    canvas_1_bitmap
};

Bool
canvas_create (Canvas * canvas, Size width, Size height, Size depth)
{
    switch (depth)
    {
    case 32:
       canvas -> canvasType = & canvas32Type;

       break;

    case 16:
       canvas -> canvasType = & canvas16Type;

       break;

    case 8:
       canvas -> canvasType = & canvas8Type;

       break;

    case 4:
       canvas -> canvasType = & canvas4Type;

       break;

    case 1:
       canvas -> canvasType = & canvas1Type;

       break;

    default:
       return FALSE;
    }
    
    canvas -> width  = width;
    canvas -> height = height;
    canvas -> depth  = depth;
    canvas -> pixels = memory_allocate ((width * height * depth + 7) / 8);

    if (canvas -> pixels == NULL)
      return FALSE;   
    
    return TRUE;
}

void
canvas_destroy (Canvas * canvas)
{
    memory_free (canvas -> pixels);
}

void     
canvas_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start , const CanvasPoint * end)
{
    (* canvas -> canvasType -> line) (canvas, color, start, end);
}

void 
canvas_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
    (* canvas -> canvasType -> horizontalLine) (canvas, color, start, length);
}

void 
canvas_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
    (* canvas -> canvasType -> verticalLine) (canvas, color, start, length);
}

void 
canvas_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
    (* canvas -> canvasType -> points) (canvas, color, points, count);
}

void 
canvas_rectangle (Canvas * canvas, CanvasColor color, const CanvasArea * area)
{
    (* canvas -> canvasType -> rectangle) (canvas, color, area);
}

void 
canvas_triangle (Canvas * canvas, CanvasColor color, const CanvasPoint * a, const CanvasPoint * b, const CanvasPoint * c)
{
    (* canvas -> canvasType -> triangle) (canvas, color, a, b, c);
}

void 
canvas_polygon (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
    (* canvas -> canvasType -> polygon) (canvas, color, points, count);
}

void
canvas_fill (Canvas * canvas, CanvasColor color)
{
    (* canvas -> canvasType -> fill) (canvas, color);
}

void 
canvas_blit (Canvas * canvas, const CanvasArea * destination, void * source)
{
    (* canvas -> canvasType -> blit) (canvas, destination, source);
}

void
canvas_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * destination, void * source)
{
    (* canvas -> canvasType -> bitmap) (canvas, color, destination, source);
}

