#ifndef __CANVAS_CANVAS_H__
#define __CANVAS_CANVAS_H__

#include <utility/types.h>
#include <canvas/color.h>
#include <canvas/point.h>
#include <canvas/area.h>

struct CanvasType;

typedef struct
{
	Size                width;
	Size                height;
	Size                depth;
	void *              pixels;
	struct CanvasType * canvasType;
} Canvas;

extern Bool	canvas_create		(Canvas *, Size, Size, Size);
extern void	canvas_destroy		(Canvas *);

extern void	canvas_line		(Canvas *, CanvasColor, const CanvasPoint *, const CanvasPoint *);
extern void	canvas_horizontal_line	(Canvas *, CanvasColor, const CanvasPoint *, Size);
extern void	canvas_vertical_line	(Canvas *, CanvasColor, const CanvasPoint *, Size);
extern void	canvas_points		(Canvas *, CanvasColor, const CanvasPoint *, Size);
extern void	canvas_rectangle	(Canvas *, CanvasColor, const CanvasArea *);
extern void	canvas_triangle		(Canvas *, CanvasColor, const CanvasPoint *, const CanvasPoint *, const CanvasPoint *);
extern void	canvas_polygon		(Canvas *, CanvasColor, const CanvasPoint *, Size);
extern void	canvas_fill		(Canvas *, CanvasColor);
extern void	canvas_blit		(Canvas *, const CanvasArea *, void *);
extern void	canvas_bitmap		(Canvas *, CanvasColor, const CanvasArea *, void *);

#endif /* __CANVAS_CANVAS_H__ */

