#include <canvas/canvas_1.h>

void
canvas_1_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels + (start -> y * canvas -> width + start -> x) / 8;
   uint32  chunk         = ~ (color - 1);
   Size currentCount;

   if (start -> x % 8)
   {
      Size offset = start -> x % 8;

      if (8 - offset > length)
      {
	 * currentMemory &= (uint8) (0xFF << (length + offset)) | ((1 << offset) - 1);
	 * currentMemory |= (uint8) ((chunk >> (32 - length)) << offset);

	 return;
      }

      * currentMemory &= (1 << offset) - 1;
      * currentMemory |= (uint8) (chunk << offset);

      ++ currentMemory;
      length -= 8 - offset;
   }

   currentCount = length / 32;
   if (currentCount > 0)
   do
   {
      * (uint32 *) currentMemory = chunk;

      currentMemory += 4;
      -- currentCount;
   } while (currentCount > 0);

   switch ((length / 8) % 4)
   {
   case 3:
      * currentMemory ++ = chunk;

   case 2:
      * currentMemory ++ = chunk;

   case 1:
      * currentMemory ++ = chunk;
   }

   if (length % 8)
   {
      * currentMemory &= (uint8) (0xFF << (length % 8));
      * currentMemory |= (uint8) (chunk & ((1 << (length % 8)) - 1));
   }
}

void
canvas_1_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels + (start -> y * canvas -> width + start -> x) / 8;
   uint8 mask = 1 << (start -> x % 8);

   color <<= start -> x % 8;

   do
   {
       * currentMemory &= mask;
       * currentMemory |= color;
       
       currentMemory += canvas -> width / 8;
   } while (-- length > 0);
}

void
canvas_1_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
   uint8 * pixels = (uint8 *) canvas -> pixels;

   while (count > 0)
   {
      uint8 pixel = pixels [(points -> y * canvas -> width + points -> x) / 8];

      pixel &= 1 << (points -> x % 8);
      pixel |= color << (points -> x % 8);
      
      pixels [(points -> y * canvas -> width + points -> x) / 8] = pixel;
      
      -- count;
      ++ points;
   }
}

void
canvas_1_blit (Canvas * canvas, const CanvasArea * area, void * data)
{
   Size currentHeight     = area -> height,
	sourceOffset      = 0,
	destinationOffset = area -> y * canvas -> width + area -> x,
	currentWidth;
   uint8 * source = (uint8 *) data,
         * destination;

   do
   {
       destination = (uint8 *) canvas -> pixels + destinationOffset / 8;

       if ((sourceOffset ^ destinationOffset) % 8)
       {
	  uint8 sourceBits = * source ++ >> (sourceOffset % 8);

	  currentWidth = area -> width;

	  while (currentWidth > 0)
          {
	       Size offset = destinationOffset % 8;

	       if ((sourceOffset % 8) > offset)
		 offset = sourceOffset % 8;
	       else
	       if (8 - offset > currentWidth)
		 sourceBits &= (1 << currentWidth) - 1;

	       * destination &= (1 << offset) - 1;
	       * destination |= sourceBits << (offset % 8);
	
	       destinationOffset += 8 - offset;
	       sourceOffset      += 8 - offset;

	       if (sourceOffset % 8)
	       {
	         sourceBits >>= 8 - offset;
		 ++ destination;
	       }
	       else
	         sourceBits = * source ++;

	       currentWidth -= 8;
	  }

	  -- currentHeight;
	  
	  continue;
       }
      
       if (sourceOffset % 8)
       {
	  * destination    &= (1 << (sourceOffset % 8)) - 1;
	  * destination ++ |= * source ++ & (0xFF << (sourceOffset % 8));
       }
       
       currentWidth = (area -> width - (sourceOffset % 8)) / 32;

       while (currentWidth > 0)
       {
	   * (uint32 *) destination = * (uint32 *) source;

	   source      += 4;
	   destination += 4;
	   -- currentWidth;
       }

       switch (((area -> width - (sourceOffset % 8)) % 32) / 8)
       {
       case 3:
	   * destination ++ = * source ++;

       case 2:
	   * destination ++ = * source ++;

       case 1:
	   * destination ++ = * source ++;
       }

       if ((area -> width ^ sourceOffset) % 8)
       {
	   Size remainder = (area -> width - (sourceOffset % 8)) % 8;

	   * destination    &= (uint8) (0xFF << remainder);
	   * destination ++ |= (uint8) (* source ++ & ((1 << remainder) - 1));
       }

       sourceOffset      += area -> width;
       destinationOffset += canvas -> width;
       
       -- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_1_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * area, void * data)
{
   Size currentHeight     = area -> height,
	sourceOffset      = 0,
	destinationOffset = area -> y * canvas -> width + area -> x,
	currentWidth;
   uint8 * source = (uint8 *) data,
         * destination;
   uint32 sourceChunk = ~ (color - 1),
          destinationChunk,
          chunkMask;

   do
   {
       destination = (uint8 *) canvas -> pixels + destinationOffset / 8;

       if ((sourceOffset ^ destinationOffset) % 8)
       {
	  uint8 sourceBits = * source ++ >> (sourceOffset % 8);

	  currentWidth = area -> width;

	  while (currentWidth > 0)
          {
	       Size offset = destinationOffset % 8;

	       if ((sourceOffset % 8) > offset)
		 offset = sourceOffset % 8;
	       else
	       if (8 - offset > currentWidth)
		 sourceBits &= (1 << currentWidth) - 1;

               chunkMask        = sourceBits << (offset % 8);
               destinationChunk = * destination;
               * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

	       destinationOffset += 8 - offset;
	       sourceOffset      += 8 - offset;

	       if (sourceOffset % 8)
	       {
	         sourceBits >>= 8 - offset;
		 ++ destination;
	       }
	       else
	         sourceBits = * source ++;

	       currentWidth -= 8;
	  }

	  -- currentHeight;
	  
	  continue;
       }
      
       if (sourceOffset % 8)
       {
          destinationChunk = * destination;
          chunkMask        = * source & (0xFF << (sourceOffset % 8));
          * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);
       }
       
       currentWidth = (area -> width - (sourceOffset % 8)) / 32;

       while (currentWidth > 0)
       {
           destinationChunk         = * (uint32 *) destination;
           chunkMask                = * (uint32 *) source;
           * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

	   source      += 4;
	   destination += 4;
	   -- currentWidth;
       }

       switch (((area -> width - (sourceOffset % 8)) % 32) / 8)
       {
       case 3:
           destinationChunk = * destination;
           chunkMask        = * source;
           * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

           ++ destination;

       case 2:
           destinationChunk = * destination;
           chunkMask        = * source;
           * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

           ++ destination;

       case 1:
           destinationChunk = * destination;
           chunkMask        = * source;
           * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

           ++ destination;
       }

       if ((area -> width ^ sourceOffset) % 8)
       {
	   Size remainder = (area -> width - (sourceOffset % 8)) % 8;

           destinationChunk = * destination;
           chunkMask        = * source & ((1 << remainder) - 1);
           * destination    = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

           ++ destination;
       }

       sourceOffset      += area -> width;
       destinationOffset += canvas -> width;
       
       -- currentHeight;
   } while (currentHeight > 0);
}

