#include <canvas/canvas_8.h>

void
canvas_16_line (Canvas * canvas, CanvasColor color, const CanvasPoint * left, const CanvasPoint * right)
{
   uint16 * currentMemory;
   Size deltaX, 
        deltaY,
	remainder,
	currentHeight;
   uint32 chunk = color |
	          (color << 16);

   if (left -> x > right -> x)
   {
      const CanvasPoint * oldLeft = left;
      
      left  = right;
      right = oldLeft;
   }

   currentMemory = (uint16 *) canvas -> pixels + left -> y * canvas -> width + left -> x;

   deltaX = right -> x - left -> x + 1;
   if (right -> y > left -> y)
     deltaY = right -> y - left -> y + 1;
   else 
     deltaY = left -> y - right -> y + 1;

   currentHeight = deltaY;
   remainder     = 0;
   do
   {
	remainder += deltaX;
       
	* currentMemory = color;
	
	while (remainder > deltaY * 2)
        {
            * (uint32 *) currentMemory = chunk;
	    
	    currentMemory += 2;
	    remainder     -= deltaY * 2;
	}
	
	while (remainder > deltaY)
        {
            * currentMemory = color;
	    
	    ++ currentMemory;
	    remainder -= deltaY;
	}

	if (left -> y < right -> y)
	  currentMemory += canvas -> width;
	else
	  currentMemory -= canvas -> width;
	
	-- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_16_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint16 * currentMemory  = (uint16 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;
   Size currentCount      = length / 2;

   if (currentCount > 0)
   {
      uint32 chunk = color |
                     (color << 16);

      do
      {
         * (uint32 *) currentMemory = chunk;

         currentMemory += 2;
         -- currentCount;
      } while (currentCount > 0);
   }

   if (length % 2)
     * currentMemory = color;
}

void
canvas_16_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint16 * currentMemory = (uint16 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;

   do
   {
       * currentMemory = color;

       currentMemory += canvas -> width;
       -- length;
   } while (length > 0);
}

void
canvas_16_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
   uint16 * pixels = (uint16 *) canvas -> pixels;

   while (count > 0)
   {
      pixels [points -> y * canvas -> width + points -> x] = color;
      
      -- count;
      ++ points;
   }
}

void
canvas_16_rectangle (Canvas * canvas, CanvasColor color, const CanvasArea * area)
{
   uint16 * currentMemory = (uint16 *) canvas -> pixels + area -> y * canvas -> width + area -> x;
   Size     stride        = canvas -> width - area -> width,
            currentHeight = area -> height,
            currentWidth;
   uint32   chunk = color |
                    (color << 16);
   
   while (currentHeight > 0)
   {
      currentWidth = area -> width / 2;

      if (currentWidth > 0)
      do
      {
         * (uint32 *) currentMemory = chunk;

         currentMemory += 2;
         -- currentWidth;
      } while (currentWidth > 0);

      if (area -> width % 2)
        * currentMemory = color;

      currentMemory += stride;
      -- currentHeight;   
   }
}

void
canvas_16_blit (Canvas * canvas, const CanvasArea * area, void * data)
{
   uint16 * destination = (uint16 *) canvas -> pixels + area -> y * canvas -> width + area -> x,
          * source      = (uint16 *) data;
   Size  stride         = canvas -> width - area -> width,
         currentHeight  = area -> height,
         currentWidth;

   while (currentHeight > 0)
   {
       currentWidth = area -> width / 2;

       while (currentWidth > 0)
       {
	   * (uint32 *) destination = * (uint32 *) source;

	   source      += 2;
	   destination += 2;
	   -- currentWidth;
       }
       
       if (area -> width % 2)
         * destination ++ = * source ++;

       destination += stride;
   }
}

void
canvas_16_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * area, void * data)
{
   static
   uint32 lowMasks [16] = { 
	                     0x00000000, 0x0000FFFF, 0xFFFF0000, 0xFFFFFFFF,
                             0x00000000, 0x0000FFFF, 0xFFFF0000, 0xFFFFFFFF,
                             0x00000000, 0x0000FFFF, 0xFFFF0000, 0xFFFFFFFF,
                             0x00000000, 0x0000FFFF, 0xFFFF0000, 0xFFFFFFFF
		          };
   static
   uint32 highMasks [16] = {
                             0x00000000, 0x00000000, 0x00000000, 0x00000000,
                             0x0000FFFF, 0x0000FFFF, 0x0000FFFF, 0x0000FFFF,
                             0xFFFF0000, 0xFFFF0000, 0xFFFF0000, 0xFFFF0000,
                             0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF
                           };
                          
   uint16 * destination = (uint16 *) canvas -> pixels + area -> y * canvas -> width + area -> x;
   uint8  * source      = (uint8 *) data;
   Size stride          = canvas -> width - area -> width,
	currentHeight   = area -> height,
	currentWidth;
   uint32 sourceChunk = color |
		        (color << 16),
          destinationChunk,
	  chunkMask;
   uint8 sourceBits; 

   while (currentHeight > 0)
   {
	currentWidth = area -> width / 8;
        
        do
	{
            sourceBits = * source;
	    while (sourceBits == 0)
            {
               destination += 8;
               
               ++ source;
               -- currentWidth;

               if (currentWidth <= 0)
                 goto nextLine;

               sourceBits = * source;
            }
            
            destinationChunk         = * (uint32 *) destination;
	    chunkMask                = lowMasks [sourceBits & 0x0F];
	    * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

	    destination += 2;           
            
            destinationChunk         = * (uint32 *) destination;
            chunkMask                = highMasks [sourceBits & 0x0F];
            * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);
            
            destination += 2;
	    
            destinationChunk         = * (uint32 *) destination;
	    chunkMask                = lowMasks [sourceBits >> 4];
	    * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

            destination += 2;

            destinationChunk         = * (uint32 *) destination;
            chunkMask                = highMasks [sourceBits >> 4];
            * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

            destination += 2;

	    ++ source;
            -- currentWidth;
        } while (currentWidth > 0);
   
   nextLine:    
	destination += stride;
	-- currentHeight;
   }	
}

