#include <canvas/canvas_8.h>

void
canvas_32_line (Canvas * canvas, CanvasColor color, const CanvasPoint * left, const CanvasPoint * right)
{
   uint32 * currentMemory;
   Size deltaX, 
        deltaY,
	remainder,
	currentHeight;

   if (left -> x > right -> x)
   {
      const CanvasPoint * oldLeft = left;
      
      left  = right;
      right = oldLeft;
   }

   currentMemory = (uint32 *) canvas -> pixels + left -> y * canvas -> width + left -> x;

   deltaX = right -> x - left -> x + 1;
   if (right -> y > left -> y)
     deltaY = right -> y - left -> y + 1;
   else 
     deltaY = left -> y - right -> y + 1;

   currentHeight = deltaY;
   remainder     = 0;
   do
   {
	remainder += deltaX;
       
	* currentMemory = color;
	
	while (remainder > deltaY)
        {
            * currentMemory = color;
	    
	    ++ currentMemory;
	    remainder -= deltaY;
	}

	if (left -> y < right -> y)
	  currentMemory += canvas -> width;
	else
	  currentMemory -= canvas -> width;
	
	-- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_32_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint32 * currentMemory  = (uint32 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;
   Size currentWidth = length / 4;

   if (currentWidth > 0)
   do
   {
      currentMemory [0] = color;
      currentMemory [1] = color;
      currentMemory [2] = color;
      currentMemory [3] = color;

      currentMemory += 4;
      -- currentWidth;
   } while (currentWidth > 0);

   switch (length % 4)
   {
   case 3:
      * currentMemory ++ = color;
      
   case 2:
      * currentMemory ++ = color;
   
   case 1:
      * currentMemory ++ = color;
   }
}

void
canvas_32_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint32 * currentMemory = (uint32 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;
   
   do
   {
       * currentMemory = color;

       currentMemory += canvas -> width;
       -- length;
   } while (length > 0);
}

void
canvas_32_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
   uint32 * pixels = (uint32 *) canvas -> pixels;

   while (count > 0)
   {
      pixels [points -> y * canvas -> width + points -> x] = color;
      
      -- count;
      ++ points;
   }
}

void
canvas_32_rectangle (Canvas * canvas, CanvasColor color, const CanvasArea * area)
{
   uint32 * currentMemory = (uint32 *) canvas -> pixels + area -> y * canvas -> width + area -> x;
   Size     stride        = canvas -> width - area -> width,
            currentHeight = area -> height,
            currentWidth;
   
   while (currentHeight > 0)
   {
       currentWidth = area -> width / 4;

       do
       {
           currentMemory [0] = color;
           currentMemory [1] = color;
           currentMemory [2] = color;
           currentMemory [3] = color;

           currentMemory += 4;
           -- currentWidth;
       } while (currentWidth > 0);

       switch (area -> width % 4)
       {
       case 3:
           * currentMemory ++ = color;

       case 2:
           * currentMemory ++ = color;
    
       case 1:
           * currentMemory ++ = color;
       }

       currentMemory += stride;
       -- currentHeight;   
   }
}

void
canvas_32_blit (Canvas * canvas, const CanvasArea * area, void * data)
{
   uint32 * destination = (uint32 *) canvas -> pixels + area -> y * canvas -> width + area -> x,
          * source      = (uint32 *) data;
   Size  stride         = canvas -> width - area -> width,
         currentHeight  = area -> height,
         currentWidth;

   while (currentHeight > 0)
   {
       currentWidth = area -> width / 4;

       do
       {
           destination [0] = source [0];
           destination [1] = source [1];
           destination [2] = source [2];
           destination [3] = source [3];

           source += 4;
           destination += 4;
           -- currentWidth;
       } while (currentWidth > 0);

       switch (area -> width % 4)
       {
       case 3:
           * destination ++ = * source ++;

       case 2:
           * destination ++ = * source ++;

       case 1:
           * destination ++ = * source ++;
       }
       
       destination += stride;
   }
}

void
canvas_32_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * area, void * data)
{
   uint32 * destination = (uint32 *) canvas -> pixels + area -> y * canvas -> width + area -> x;
   uint8 * source       = (uint8 *) data;
   Size stride          = canvas -> width - area -> width,
	currentHeight   = area -> height,
	currentWidth;
   uint8 sourceBits; 

   while (currentHeight > 0)
   {
	currentWidth = area -> width / 8;
        
        do
	{
            sourceBits = * source;
	    while (sourceBits == 0)
            {
               destination += 8;
               
               ++ source;
               -- currentWidth;

               if (currentWidth <= 0)
                 goto nextLine;

               sourceBits = * source;
            }
           
            if (sourceBits & (1 << 0))
              destination [0] = color;
            if (sourceBits & (1 << 1))
              destination [1] = color;
            if (sourceBits & (1 << 2))
              destination [2] = color;
            if (sourceBits & (1 << 3))
              destination [3] = color;
            if (sourceBits & (1 << 4))
              destination [4] = color;
            if (sourceBits & (1 << 5))
              destination [5] = color;
            if (sourceBits & (1 << 6))
              destination [6] = color;
            if (sourceBits & (1 << 7))
              destination [7] = color;

            destination += 8;
	    ++ source;
            -- currentWidth;
        } while (currentWidth > 0);
   
   nextLine:    
	destination += stride;
	-- currentHeight;
   }	
}

