#include <canvas/canvas_4.h>

void
canvas_4_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels + (start -> y * canvas -> width + start -> x) / 2;
   uint32  chunk         = (uint8) color | ((uint8) color << 4);
   Size currentCount;

   chunk |= chunk << 8;
   chunk |= chunk << 16;

   if (start -> x % 2)
   {
      * currentMemory &= 0x0F;
      * currentMemory |= (color << 4);

      ++ currentMemory;
      -- length;
   }

   currentCount = length / 8;
   if (currentCount > 0)
   do
   {
      * (uint32 *) currentMemory = chunk;

      currentMemory += 4;
      -- currentCount;
   } while (currentCount > 0);

   switch ((length % 8) / 2)
   {
   case 3:
      * currentMemory ++ = chunk;

   case 2:
      * currentMemory ++ = chunk;

   case 1:
      * currentMemory ++ = chunk;
   }

   if (length % 2)
   {
      * currentMemory &= 0xF0;
      * currentMemory |= color;
   }
}

void
canvas_4_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels + (start -> y * canvas -> width + start -> x) / 2;
   
   if (start -> x % 2)
   do
   {
       * currentMemory &= 0x0F;
       * currentMemory |= color << 4;

       currentMemory += canvas -> width / 2;
   } while (-- length > 0);
   else
   do
   {
       * currentMemory &= 0xF0;
       * currentMemory |= color;

       currentMemory += canvas -> width / 2;
   } while (-- length > 0);
}

void
canvas_4_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
   uint8 * pixels = (uint8 *) canvas -> pixels;

   while (count > 0)
   {
      uint8 pixel = pixels [(points -> y * canvas -> width + points -> x) / 2];

      if (points -> x % 2)
      {
         pixel &= 0x0F;
	 pixel |= color << 4;
      }
      else
      {
	 pixel &= 0xF0;
	 pixel |= color;
      }
      
      pixels [(points -> y * canvas -> width + points -> x) / 2] = pixel;
      
      -- count;
      ++ points;
   }
}

void
canvas_4_blit (Canvas * canvas, const CanvasArea * area, void * data)
{
   Size currentHeight     = area -> height,
	sourceOffset      = 0,
	destinationOffset = area -> y * canvas -> width + area -> x,
	currentWidth;
   uint8 * source = (uint8 *) data,
         * destination;
   uint32 sourceChunk;

   do
   {
       destination = (uint8 *) canvas -> pixels + destinationOffset / 2;

       if ((sourceOffset ^ destinationOffset) % 2)
       {
	  currentWidth = area -> width;

          sourceChunk = * source;

          if (destinationOffset % 2)
          {
             * destination = (* destination & 0x0F) | (sourceChunk << 4);

             ++ destination;

             ++ sourceOffset;
             -- currentWidth;
          }
          else
            sourceChunk = * source;
         
          sourceChunk  >>= 4;
          sourceOffset  += currentWidth / 2;

          while (currentWidth > 1)
          {
               sourceChunk   |= (* source << 4);
               * destination  = sourceChunk;

               ++ source;
               ++ destination;

               sourceChunk  >>= 8;
               currentWidth  -= 2;
          }

          if (currentWidth > 0)
          {
               * destination = (* destination & 0xF0) | sourceChunk;

               ++ source;
               ++ sourceOffset;
          } 

	  destinationOffset += canvas -> width;
	  -- currentHeight;
	  
	  continue;
       }
      
       if (sourceOffset % 2)
       {
	  * destination    &= 0x0F;
	  * destination ++ |= * source ++ & 0xF0;
       }
       
       currentWidth = (area -> width - (sourceOffset % 2)) / 8;

       while (currentWidth > 0)
       {
	   * (uint32 *) destination = * (uint32 *) source;

	   source      += 4;
	   destination += 4;
	   -- currentWidth;
       }

       switch (((area -> width - (sourceOffset % 2)) % 8) / 2)
       {
       case 3:
	   * destination ++ = * source ++;

       case 2:
	   * destination ++ = * source ++;

       case 1:
	   * destination ++ = * source ++;
       }

       if ((area -> width ^ sourceOffset) % 2)
       {
	   * destination    &= 0xF0;
	   * destination ++ |= * source ++;
       }

       sourceOffset      += area -> width;
       destinationOffset += canvas -> width;
       
       -- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_4_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * area, void * data)
{
   static
   uint16 masks [16] = {
	                 0x0000, 0x000F, 0x00F0, 0x00FF,
			 0x0F00, 0x0F0F, 0x0FF0, 0x0FFF,
			 0xF000, 0xF00F, 0xF0F0, 0xF0FF,
			 0xFF00, 0xFF0F, 0xFFF0, 0xFFFF
		       };
   uint8 * source = (uint8 *) data,
         * destination;
   Size destinationOffset = area -> y * canvas -> width + area -> x,
        currentHeight     = area -> height,
	currentWidth;
   uint32 sourceChunk = color | (color << 4),
          destinationChunk,
	  chunkMask;
   uint8 sourceBits;

   sourceChunk |= sourceChunk << 8;
   sourceChunk |= sourceChunk << 16;

   while (currentHeight > 0)
   {
        destination  = (uint8 *) canvas -> pixels + destinationOffset / 2;
	currentWidth = area -> width / 8;

	if (destinationOffset % 2)
	do
	{
	    sourceBits = * source;

	    destinationChunk          = * (uint32 *) destination;
	    chunkMask                 = masks [sourceBits & 0x0F] << 4;
	    chunkMask                |= masks [sourceBits >> 4] << 20;
	    * (uint32 *) destination  = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

	    destination += 4;

            destinationChunk = * destination;
            chunkMask        = masks [sourceBits >> 4] >> 12;
            * destination    = (destinationChunk & 0xF0) | (color & chunkMask);
	    
	    ++ source;
	    -- currentWidth;
	} while (currentWidth > 0);
	else
        do
	{
	    sourceBits = * source;
	    
	    destinationChunk          = * (uint32 *) destination;
	    chunkMask                 = masks [sourceBits & 0x0F];
	    chunkMask                |= masks [sourceBits >> 4] << 16;
	    * (uint32 *) destination  = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

            destination += 4;
	    ++ source;
	    -- currentWidth;
	} while (currentWidth > 0);

	destinationOffset += area -> width;
	-- currentHeight;
   }
}

