#include <canvas/canvas_8.h>

void
canvas_8_line (Canvas * canvas, CanvasColor color, const CanvasPoint * left, const CanvasPoint * right)
{
   uint8 * currentMemory;
   Size deltaX, 
        deltaY,
	remainder,
	currentHeight;
   uint32 chunk = color |
	          (color << 8) |
		  (color << 16) |
		  (color << 24);

   if (left -> x > right -> x)
   {
      const CanvasPoint * oldLeft = left;
      
      left  = right;
      right = oldLeft;
   }

   currentMemory = (uint8 *) canvas -> pixels + left -> y * canvas -> width + left -> x;

   deltaX = right -> x - left -> x + 1;
   if (right -> y > left -> y)
     deltaY = right -> y - left -> y + 1;
   else 
     deltaY = left -> y - right -> y + 1;

   currentHeight = deltaY;
   remainder     = 0;
   do
   {
	remainder += deltaX;
       
	* currentMemory = color;
	
	while (remainder > deltaY * 4)
        {
            * (uint32 *) currentMemory = chunk;
	    
	    currentMemory += 4;
	    remainder     -= deltaY * 4;
	}
	
	while (remainder > deltaY)
        {
            * currentMemory = color;
	    
	    ++ currentMemory;
	    remainder -= deltaY;
	}

	if (left -> y < right -> y)
	  currentMemory += canvas -> width;
	else
	  currentMemory -= canvas -> width;
	
	-- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_8_horizontal_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory  = (uint8 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;
   Size currentCount      = length / 4;

   if (currentCount > 0)
   {
      uint32 chunk = color |
                     (color << 8) |
                     (color << 16) |
                     (color << 24);

      do
      {
         * (uint32 *) currentMemory = chunk;

         currentMemory += 4;
         -- currentCount;
      } while (currentCount > 0);
   }

   switch (length % 4)
   {
   case 3:
      * currentMemory ++ = color;

   case 2:
      * currentMemory ++ = color;

   case 1:
      * currentMemory    = color;
   }
}

void
canvas_8_vertical_line (Canvas * canvas, CanvasColor color, const CanvasPoint * start, Size length)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels  + start -> y * canvas -> width + start -> x;

   do
   {
       * currentMemory = color;

       currentMemory += canvas -> width;
   } while (-- length > 0);
}

void
canvas_8_points (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
   uint8 * pixels = (uint8 *) canvas -> pixels;

   while (count > 0)
   {
      pixels [points -> y * canvas -> width + points -> x] = color;
      
      -- count;
      ++ points;
   }
}

void
canvas_8_rectangle (Canvas * canvas, CanvasColor color, const CanvasArea * area)
{
   uint8 * currentMemory = (uint8 *) canvas -> pixels + area -> y * canvas -> width + area -> x;
   Size    stride        = canvas -> width - area -> width,
           currentHeight = area -> height,
           currentWidth;
   uint32  chunk = color |
                   (color << 8) |
                   (color << 16) |
	           (color << 24);
   
   while (currentHeight > 0)
   {
      currentWidth = area -> width / 4;

      if (currentWidth > 0)
      do
      {
         * (uint32 *) currentMemory = chunk;

         currentMemory += 4;
         -- currentWidth;
      } while (currentWidth > 0);

      switch (area -> width % 4)
      {
      case 3:
         * currentMemory ++ = color;

      case 2:
         * currentMemory ++ = color;

      case 1:
         * currentMemory ++ = color;
      }

      currentMemory += stride;
      -- currentHeight;   
   }
}

void
canvas_8_blit (Canvas * canvas, const CanvasArea * area, void * data)
{
   uint8 * destination = (uint8 *) canvas -> pixels + area -> y * canvas -> width + area -> x,
         * source      = (uint8 *) data;
   Size stride         = canvas -> width - area -> width,
        currentHeight  = area -> height,
        currentWidth;

   while (currentHeight > 0)
   {
       currentWidth = area -> width / 4;

       while (currentWidth > 0)
       {
	   * (uint32 *) destination = * (uint32 *) source;

	   source      += 4;
	   destination += 4;
	   -- currentWidth;
       }
       
       switch (area -> width % 4)
       {
       case 3:
	   * destination ++ = * source ++;

       case 2:
	   * destination ++ = * source ++;

       case 1:
	   * destination ++ = * source ++;
       }

       destination += stride;
   }
}

void
canvas_8_bitmap (Canvas * canvas, CanvasColor color, const CanvasArea * area, void * data)
{
   static
   uint32 masks [16] = { 
	                 0x00000000, 0x000000FF, 0x0000FF00, 0x0000FFFF,
	                 0x00FF0000, 0x00FF00FF, 0x00FFFF00, 0x00FFFFFF,
			 0xFF000000, 0xFF0000FF, 0xFF00FF00, 0xFF00FFFF,
			 0xFFFF0000, 0xFFFF00FF, 0xFFFFFF00, 0xFFFFFFFF
		       };
   uint8 * destination = (uint8 *) canvas -> pixels + area -> y * canvas -> width + area -> x,
         * source      = (uint8 *) data;
   Size stride         = canvas -> width - area -> width,
	currentHeight  = area -> height,
	currentWidth;
   uint32 sourceChunk = color |
	                (color << 8) |
		        (color << 16) |
		        (color << 24),
          destinationChunk,
	  chunkMask;
   uint8 sourceBits; 

   while (currentHeight > 0)
   {
	currentWidth = area -> width / 8;
        
        do
	{
            sourceBits = * source;
	    while (sourceBits == 0)
            {
                destination += 8;

                ++ source;
                -- currentWidth;
                
                if (currentWidth <= 0)
                  goto nextLine;
                
                sourceBits = * source;
            }
            
            destinationChunk         = * (uint32 *) destination;
	    chunkMask                = masks [sourceBits & 0x0F];
	    * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

	    destination += 4;
	    
	    destinationChunk         = * (uint32 *) destination;
	    chunkMask                = masks [sourceBits >> 4];
	    * (uint32 *) destination = (destinationChunk & ~ chunkMask) | (sourceChunk & chunkMask);

            destination += 4;

	    ++ source;
            -- currentWidth;
        } while (currentWidth > 0);
   
   nextLine:
	destination += stride;
	-- currentHeight;
   }	
}

