#include <canvas/canvas_default.h>

void
canvas_default_line (Canvas * canvas, CanvasColor color, const CanvasPoint * left, const CanvasPoint * right)
{
   CanvasPoint currentPoint;
   Size deltaX, 
        deltaY,
	remainder,
	currentHeight;
   
   if (left -> x > right -> x)
   {
      const CanvasPoint * oldLeft = left;

      left  = right;
      right = oldLeft;
   }

   currentPoint = * left;

   deltaX = right -> x - left -> x + 1;
   if (right -> y > left -> y)
     deltaY = right -> y - left -> y + 1;
   else
     deltaY = left -> y - right -> y + 1;

   currentHeight = deltaY;
   remainder     = 0;
   do
   {
	Size length = 0;

	for (remainder += deltaX;
	     remainder > deltaY;
	     remainder -= deltaY)
           ++ length;
	
	(* canvas -> canvasType -> horizontalLine) (canvas,
						    color,
						    & currentPoint,
						    (length > 0) ? length : 1);

	currentPoint.x += length;
	if (left -> y < right -> y)
          -- currentPoint.y;
	else
	  ++ currentPoint.y;

	-- currentHeight;
   } while (currentHeight > 0);
}

void
canvas_default_rectangle (Canvas * canvas, CanvasColor color, const CanvasArea * area)
{
   CanvasPoint currentPoint = { area -> x, area -> y };
   Size currentHeight = area -> height;
   
   while (currentHeight > 0)
   {
      (* canvas -> canvasType -> horizontalLine) (canvas, color, & currentPoint, area -> width);

      ++ currentPoint.y;
      -- currentHeight;
   }
}

void
canvas_default_triangle (Canvas * canvas, CanvasColor color, const CanvasPoint * a, const CanvasPoint * b, const CanvasPoint * c)
{
   CanvasPoint currentLeft;
   int leftDeltaX, 
       rightDeltaX,
       leftRemainder  = 0,
       rightRemainder = 0;
   Size currentRight,
        currentEnd,
	leftDeltaY,
        rightDeltaY;

   if (b -> y < a -> y)
   {
       const CanvasPoint * oldA = a;

       a = b;
       b = oldA;
   }
   if (c -> y < b -> y)
   {
       const CanvasPoint * oldB = b;

       b = c;
       c = oldB;
   }
   else
   if (c -> y < a -> y)
   {
       const CanvasPoint * oldA = a;

       a = c;
       c = b;
       b = oldA;
   }

   currentLeft  = * a;
   currentRight = a -> x;
   currentEnd   = b -> y;

   if (b -> x < c -> x)
   {
       leftDeltaX  = b -> x - a -> x;
       leftDeltaY  = b -> y - a -> y + 1; 
       rightDeltaX = c -> x - a -> x + 1;
       rightDeltaY = c -> y - a -> y + 1;
   }
   else
   {
       leftDeltaX  = c -> x - a -> x;
       leftDeltaY  = c -> y - a -> y + 1;
       rightDeltaX = b -> x - a -> x + 1;
       rightDeltaY = b -> y - a -> y + 1;
   }

   for (;;)
   {
      do
      {
          if (leftDeltaX < 0)
          for (leftRemainder -= leftDeltaX;
	       leftRemainder > leftDeltaY;
	       leftRemainder -= leftDeltaY)
	     -- currentLeft.x;
          else
          for (leftRemainder += leftDeltaX;
  	       leftRemainder > leftDeltaY;
	       leftRemainder -= leftDeltaY)
	     ++ currentLeft.x;

          if (rightDeltaX < 0)
          for (rightRemainder -= rightDeltaX;
	       rightRemainder > rightDeltaY;
	       rightRemainder -= rightDeltaY)
	     -- currentRight;
          else
          for (rightRemainder += rightDeltaX;
               rightRemainder > rightDeltaY;
	       rightRemainder -= rightDeltaY)
	     ++ currentRight;

          (* canvas -> canvasType -> horizontalLine) (canvas, 
	    		  		  	      color, 
						      & currentLeft,
						      currentRight - currentLeft.x);
          ++ currentLeft.y;
      } while (currentLeft.y <= currentEnd);

      if (currentLeft.y > c -> y)
	break;

      if (b -> x < c -> x)
      {
          leftDeltaX = c -> x - b -> x;
	  leftDeltaY = c -> y - b -> y + 1;
      }
      else
      {
	  rightDeltaX = c -> x - b -> x + 1;
	  rightDeltaY = c -> y - b -> y + 1;
      }
   }
}

void
canvas_default_polygon (Canvas * canvas, CanvasColor color, const CanvasPoint * points, Size count)
{
    const CanvasPoint * pivot = points;

    while (-- count >= 2)
    {
        ++ points;
        
        (* canvas -> canvasType -> triangle) (canvas,
					      color,
					      pivot,
					      points,
					      & points [1]);
    }
}

void
canvas_default_fill (Canvas * canvas, CanvasColor color)
{
    CanvasArea area;

    area.x      = 0;
    area.y      = 0;
    area.width  = canvas -> width;
    area.height = canvas -> height;

    (* canvas -> canvasType -> rectangle) (canvas, color, & area);
}

