#ifndef __CONSOLE_CONSOLE_H__
#define __CONSOLE_CONSOLE_H__

#include <utility/types.h>
#include <utility/arguments.h>
#include <console/color.h>

struct ConsoleType;

typedef Size ConsoleCoordinate;
typedef int  ConsoleOffset;

typedef struct
{
        struct ConsoleType *    type;
        ConsoleColor            foregroundColor;
        ConsoleColor            backgroundColor;
        ConsoleCoordinate       xCursor;
        ConsoleCoordinate       yCursor;
        ConsoleCoordinate       xSize;
        ConsoleCoordinate       ySize;
        ConsoleCoordinate       xVirtualOrigin;
        ConsoleCoordinate       yVirtualOrigin;
        ConsoleCoordinate       xVirtualSize;
        ConsoleCoordinate       yVirtualSize;
        uint16 *                textMemory;
        void *                  data;
} Console;

extern Bool      console_create         (Console **, struct ConsoleType *, ConsoleCoordinate, ConsoleCoordinate);
extern void      console_destroy        (Console *);

extern Console * console_current        (void);
extern void      console_activate       (Console *);
extern void      console_render         (Console *);

extern void      console_set_cursor     (Console *, ConsoleCoordinate, ConsoleCoordinate);
extern void      console_set_origin     (Console *, ConsoleCoordinate, ConsoleCoordinate);

extern void      console_set_foreground_color (Console *, ConsoleColor);
extern void      console_set_background_color (Console *, ConsoleColor);

extern void      console_tab             (Console *);
extern void      console_newline         (Console *);
extern void      console_print           (Console *, const char *);
extern void      console_print_format    (Console *, const char *, ...);
extern void      console_print_arguments (Console *, const char *, Arguments);

#endif /* __CONSOLE_CONSOLE_H__ */

