#include <utility/memory.h>
#include <memory/memory.h>
#include <vga/context.h>
#include <console/console_vga.h>

static ConsoleType consoleVGAType =
{
        console_vga_create,
        console_vga_destroy,
        console_vga_activate,
        console_vga_deactivate,
        console_vga_set_cursor,
        console_vga_render_line
};

ConsoleType * CONSOLE_TYPE_VGA     = & consoleVGAType;
ConsoleType * CONSOLE_TYPE_DEFAULT = & consoleVGAType;

Bool
console_vga_create (Console * console, ConsoleCoordinate xVirtualSize, ConsoleCoordinate yVirtualSize)
{
        if (console_current () == NULL)
          console -> data = vga_context_current ();
        else
        {
           console -> data = memory_allocate (sizeof (VGAContext));

           if (console -> data == NULL)
             return FALSE;
           
           vga_context_create ((VGAContext *) console -> data,
                               VGA_MODE_TEXT_80x25,
                               NULL,
                               NULL,
                               NULL,
                               NULL);
        }
        
        console -> xSize = 80;
        console -> ySize = 25;

        return TRUE;
}

void
console_vga_destroy (Console * console)
{
        vga_context_destroy ((VGAContext *) console -> data);
        
        memory_free (console -> data);
}

void
console_vga_activate (Console * console)
{
        vga_context_activate ((VGAContext *) console -> data);
}

void
console_vga_deactivate (Console * console)
{
}

void
console_vga_set_cursor (Console * console, 
                        ConsoleCoordinate x,
                        ConsoleCoordinate y)
{
        vga_context_set_cursor ((VGAContext *) console -> data, x, y);
}

void
console_vga_render_line (Console * console,
                         ConsoleCoordinate y,
                         Size length,
                         const uint16 * text)
{
        VGAContext * context = (VGAContext *) console -> data;

        memory_copy ((uint16 *) context -> videoMemory + y * 80,
                     text,
                     length * sizeof (uint16));

        if (length < 80)
          memory_zero ((uint16 *) context -> videoMemory + y * 80 + length,
                       (80 - length) * sizeof (uint16));
}


