#ifndef __DMA_DMA_H__
#define __DMA_DMA_H__

#include <utility/types.h>

enum
{
   DMA_CHANNEL_COUNT = 8
};

enum
{
   DMA_ADDRESS_LIMIT = 0x100000
};

enum
{
   DMA_COUNT_LIMIT = 0x10000
};

typedef enum
{
   DMA_REGISTER_MASK_8   = 0x0A,
   DMA_REGISTER_MODE_8   = 0x0B,
   DMA_REGISTER_CLEAR_8  = 0x0C,
   
   DMA_REGISTER_MASK_16  = 0xD4,
   DMA_REGISTER_MODE_16  = 0xD6,
   DMA_REGISTER_CLEAR_16 = 0xD8,

   DMA_REGISTER_CHANNEL_0_PAGE = 0x87,
   DMA_REGISTER_CHANNEL_1_PAGE = 0x83,
   DMA_REGISTER_CHANNEL_2_PAGE = 0x81,
   DMA_REGISTER_CHANNEL_3_PAGE = 0x82,
   DMA_REGISTER_CHANNEL_4_PAGE = 0x8F,
   DMA_REGISTER_CHANNEL_5_PAGE = 0x8B,
   DMA_REGISTER_CHANNEL_6_PAGE = 0x89,
   DMA_REGISTER_CHANNEL_7_PAGE = 0x8A,

   DMA_REGISTER_CHANNEL_0_ADDRESS = 0x00,
   DMA_REGISTER_CHANNEL_1_ADDRESS = 0x02,
   DMA_REGISTER_CHANNEL_2_ADDRESS = 0x04,
   DMA_REGISTER_CHANNEL_3_ADDRESS = 0x06,
   DMA_REGISTER_CHANNEL_4_ADDRESS = 0xC0,
   DMA_REGISTER_CHANNEL_5_ADDRESS = 0xC4,
   DMA_REGISTER_CHANNEL_6_ADDRESS = 0xC8,
   DMA_REGISTER_CHANNEL_7_ADDRESS = 0xCC,

   DMA_REGISTER_CHANNEL_0_COUNT = 0x01,
   DMA_REGISTER_CHANNEL_1_COUNT = 0x03,
   DMA_REGISTER_CHANNEL_2_COUNT = 0x05,
   DMA_REGISTER_CHANNEL_3_COUNT = 0x07,
   DMA_REGISTER_CHANNEL_4_COUNT = 0xC2,
   DMA_REGISTER_CHANNEL_5_COUNT = 0xC6,
   DMA_REGISTER_CHANNEL_6_COUNT = 0xCA,
   DMA_REGISTER_CHANNEL_7_COUNT = 0xCE
} DMARegister;

typedef enum
{
   DMA_COMMAND_CHANNEL_0      = 0x00,
   DMA_COMMAND_CHANNEL_1      = 0x01,
   DMA_COMMAND_CHANNEL_2      = 0x02,
   DMA_COMMAND_CHANNEL_3      = 0x03,

   DMA_COMMAND_CHANNEL_4      = 0x00,
   DMA_COMMAND_CHANNEL_5      = 0x01,
   DMA_COMMAND_CHANNEL_6      = 0x02,
   DMA_COMMAND_CHANNEL_7      = 0x03,

   DMA_COMMAND_MASK_CLEAR     = 0x00,
   DMA_COMMAND_MASK_SET       = 0x04,

   DMA_COMMAND_MODE_VERIFY    = 0x00,
   DMA_COMMAND_MODE_READ      = 0x04,
   DMA_COMMAND_MODE_WRITE     = 0x08,

   DMA_COMMAND_MODE_MANUAL    = 0x00,
   DMA_COMMAND_MODE_AUTO      = 0x10,

   DMA_COMMAND_MODE_DECREMENT = 0x00,
   DMA_COMMAND_MODE_INCREMENT = 0x20,

   DMA_COMMAND_MODE_DEMAND    = 0x00,
   DMA_COMMAND_MODE_SINGLE    = 0x40,
   DMA_COMMAND_MODE_BLOCK     = 0x80,
   DMA_COMMAND_MODE_CASCADE   = 0xB0,
   
   DMA_COMMAND_CLEAR          = 0x00
} DMACommand;
   
enum
{
   DMA_CHANNEL_NUMBER_ANY = ~0
};

typedef unsigned DMAChannelNumber;

typedef enum
{
   DMA_CHANNEL_ATTRIBUTE_8   = (1 << 0),
   DMA_CHANNEL_ATTRIBUTE_16  = (1 << 1),
   DMA_CHANNEL_ATTRIBUTE_ANY = ~0
} DMAChannelAttribute;

typedef FlagSet DMAChannelAttributes;

typedef struct
{
        Bool                 isAllocated;

        DMARegister          maskRegister;
        DMARegister          modeRegister;
        DMARegister          clearRegister;

        DMARegister          pageRegister;
        DMARegister          addressRegister;
        DMARegister          countRegister;

        DMACommand           commandModifier;

        DMAChannelAttributes attributes;
} DMAChannel;

typedef enum
{
   DMA_TRANSFER_READ  = DMA_COMMAND_MODE_READ,
   DMA_TRANSFER_WRITE = DMA_COMMAND_MODE_WRITE
} DMATransfer;

extern const DMAChannel * dma_channel_allocate (DMAChannelNumber, DMAChannelAttributes);
extern void dma_channel_free    (const DMAChannel *);

extern Bool dma_transfer_start  (const DMAChannel *, DMATransfer, void *, Size);
extern void dma_transfer_stop   (const DMAChannel *);

extern void * dma_buffer_allocate (Size);
extern void dma_buffer_free (void *);

#endif /* __DMA_DMA_H__ */

