%include "utility/assembly.i"
%include "i386/boot.i"
%include "i386/manage.i"
%include "i386/general.i"

section .text read nowrite exec contents

	extern console_print_format
        extern console_print
	extern thread_yield
        extern xcom_boot

DATA boot_information
	istruc i386BootInformation
		at i386BootInformation.magic,		dd 0
		at i386BootInformation.imageSize, 	dd 0
		at i386BootInformation.loadAddress,	dd 0
		at i386BootInformation.memorySize,	dd 0
	iend
DATA_END


        global _start
_start:
        mov	esp, I386_STACK_OFFSET + I386_STACK_LIMIT

        push    dword I386_EFLAGS_FLAG_RESERVED
        popf

        mov     edi, I386_GDT_OFFSET
        mov     ecx, I386_GDT_LIMIT / 4
        xor     eax, eax
        rep     stosd

        mov     edi, I386_GDT_OFFSET + I386_SELECTOR_TSS
        mov     esi, globalDescriptorTable
        mov     ecx, (I386_SELECTOR_USER_DATA + 8 - I386_SELECTOR_TSS) / 4
        rep     movsd

        sub     esp, 8
        mov     [esp], word I386_GDT_LIMIT
        mov     [esp + 2], dword I386_GDT_OFFSET
        lgdt    [esp]
        add     esp, 8

        mov     ax, I386_SELECTOR_SYSTEM_DATA
        mov     ss, ax
        mov     ds, ax
        mov     es, ax
        mov     fs, ax
        mov     gs, ax

        jmp     I386_SELECTOR_SYSTEM_CODE : .loadCS

.loadCS:
        mov     edi, I386_TSS_OFFSET
        mov     esi, taskStateSegment
        mov     ecx, i386TaskStateSegment.size / 4
        rep     movsd
      
        mov     ax, I386_SELECTOR_TSS
        ltr     ax

	mov	ebx, [boot_information + i386BootInformation.loadAddress]
	mov	ecx, [boot_information + i386BootInformation.imageSize]

	cmp	[ebx + i386BootHeader.magic], dword I386_BOOT_MAGIC
	jne	.loadXCom

	add	ebx, i386BootHeader.size
	sub	ecx, i386BootHeader.size

.loadXCom:
        push    dword [ebx + XComHeader.fileSize]
        push    ebx
        push    dword 0
        call    xcom_boot
        add     esp, 3 * 4

        test    eax, eax
        jz      .bootFailure

	push	dword 0
	call	thread_yield
	add	esp, 4

.bootFailure:
	jmp	$


globalDescriptorTable:
; TSS (0x0018)
        dw      I386_TSS_LIMIT - 1
        dw      I386_TSS_OFFSET
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_TYPE_TSS | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0
        db      0

; system code segment (0x0020)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_CODE | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; system data segment (0x0028)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; user code segment (0x0030)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_CODE | I386_DESCRIPTOR_ATTRIBUTE_DPL_3 | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0

; user data segment (0x0038)
        dw      0xFFFF
        dw      0x0000
        db      0
        db      I386_DESCRIPTOR_ATTRIBUTE_READ_ENABLE | I386_DESCRIPTOR_ATTRIBUTE_NON_SYSTEM | I386_DESCRIPTOR_ATTRIBUTE_DPL_3 | I386_DESCRIPTOR_ATTRIBUTE_PRESENT
        db      0xF | I386_DESCRIPTOR_ATTRIBUTE_PAGE_GRANULARITY | I386_DESCRIPTOR_ATTRIBUTE_32_BIT
        db      0


taskStateSegment:
        istruc i386TaskStateSegment
                at i386TaskStateSegment.backLink,       dd 0
                at i386TaskStateSegment.ESP_0,          dd I386_STACK_OFFSET + I386_STACK_LIMIT
                at i386TaskStateSegment.SS_0,           dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.ESP_1,          dd I386_STACK_OFFSET + I386_STACK_LIMIT
                at i386TaskStateSegment.SS_1,           dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.ESP_2,          dd I386_STACK_OFFSET + I386_STACK_LIMIT
                at i386TaskStateSegment.SS_2,           dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.CR3,            dd 0
                at i386TaskStateSegment.EIP,            dd 0
                at i386TaskStateSegment.EFLAGS,         dd I386_EFLAGS_FLAG_RESERVED
                at i386TaskStateSegment.EAX,            dd 0
                at i386TaskStateSegment.ECX,            dd 0
                at i386TaskStateSegment.EDX,            dd 0
                at i386TaskStateSegment.EBX,            dd 0
                at i386TaskStateSegment.ESP,            dd I386_STACK_OFFSET + I386_STACK_LIMIT
                at i386TaskStateSegment.EBP,            dd 0
                at i386TaskStateSegment.ESI,            dd 0
                at i386TaskStateSegment.EDI,            dd 0
                at i386TaskStateSegment.ES,             dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.CS,             dd I386_SELECTOR_SYSTEM_CODE
                at i386TaskStateSegment.SS,             dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.DS,             dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.FS,             dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.GS,             dd I386_SELECTOR_SYSTEM_DATA
                at i386TaskStateSegment.LDTR,           dd 0
                at i386TaskStateSegment.debugTrap,      dw 0
                at i386TaskStateSegment.ioBitmap,       dw 0x1000
        iend

