#ifndef __I386_IO_H__
#define __I386_IO_H__

#include <utility/types.h>
#include <utility/inline.h>

INLINE uint8 	i386_read_8 	(void * address)
{ return * (volatile uint8 *) address; }

INLINE uint16 	i386_read_16 	(void * address)
{ return * (volatile uint16 *) address; }

INLINE uint32 	i386_read_32	(void * address)
{ return * (volatile uint32 *) address; }

INLINE void	i386_write_8	(void * address, uint8 value)
{ * (volatile uint8 *) address = value; }

INLINE void     i386_write_16   (void * address, uint16 value)
{ * (volatile uint16 *) address = value; }

INLINE void     i386_write_32   (void * address, uint32 value)
{ * (volatile uint32 *) address = value; }

#ifdef __GNUC__
#include <utility/assembly.h>

INLINE uint8	i386_input_8	(uint16 port)
{
   uint8 value;

   ASSEMBLY ("inb %1, %0" : "=a" (value) : "d" (port));
   
   return value;
}

INLINE uint16    i386_input_16    (uint16 port)
{
   uint16 value;

   ASSEMBLY ("inw %1, %0" : "=a" (value) : "d" (port));

   return value;
}

INLINE uint32    i386_input_32   (uint16 port)
{
   uint32 value;

   ASSEMBLY ("inl %1, %0" : "=a" (value) : "d" (port));

   return value;
}

INLINE void	i386_output_8	(uint16 port, uint8 value)
{ ASSEMBLY ("outb %0, %1" :: "a" (value), "d" (port)); }

INLINE void     i386_output_16   (uint16 port, uint16 value)
{ ASSEMBLY ("outw %0, %1" :: "a" (value), "d" (port)); }

INLINE void     i386_output_32   (uint16 port, uint32 value)
{ ASSEMBLY ("outl %0, %1" :: "a" (value), "d" (port)); }

#else /* ! __GNUC__ */

extern uint8    i386_input_8    (uint16);
extern uint16   i386_input_16   (uint16);
extern uint32   i386_input_32   (uint16);

extern void     i386_output_8   (uint16, uint8);
extern void     i386_output_16  (uint16, uint16);
extern void     i386_output_32  (uint16, uint32);

#endif /* __GNUC__ */

#endif /* __I386_IO_H__ */


