#ifndef __I386_PAGE_H__
#define __I386_PAGE_H__

#include <utility/types.h>

typedef enum
{
   I386_PAGE_BITS      = 12,
   I386_PAGE_SHIFT     = 0,
   I386_PAGE_ALIGNMENT = I386_PAGE_BITS + I386_PAGE_SHIFT,
   
   I386_PAGE_SIZE      = 4096,

   I386_PAGE_MASK      = (1 << I386_PAGE_ALIGNMENT) - 1
} i386PageAttributes;

typedef enum
{
   I386_PAGE_TABLE_BITS      = 10,
   I386_PAGE_TABLE_SHIFT     = I386_PAGE_ALIGNMENT,
   I386_PAGE_TABLE_ALIGNMENT = I386_PAGE_TABLE_BITS + I386_PAGE_TABLE_SHIFT,

   I386_PAGE_TABLE_ENTRIES   = (1 << I386_PAGE_TABLE_BITS),

   I386_PAGE_TABLE_MASK      = (1 << I386_PAGE_TABLE_ALIGNMENT) - 1
} i386PageTableAttributes;

typedef enum
{
   I386_PAGE_DIRECTORY_BITS      = 10,
   I386_PAGE_DIRECTORY_SHIFT     = I386_PAGE_TABLE_ALIGNMENT,
   I386_PAGE_DIRECTORY_ALIGNMENT = I386_PAGE_DIRECTORY_BITS + I386_PAGE_DIRECTORY_SHIFT,
  
   I386_PAGE_DIRECTORY_ENTRIES   = (1 << I386_PAGE_DIRECTORY_BITS),

   I386_PAGE_DIRECTORY_MASK      = ~0
} i386PageDirectoryAttributes;

typedef enum
{
   I386_PAGE_ENTRY_PRESENT       = (1 << 0),
   I386_PAGE_ENTRY_WRITEABLE     = (1 << 1),
   I386_PAGE_ENTRY_USER          = (1 << 2),
   I386_PAGE_ENTRY_WRITE_THROUGH = (1 << 3),
   I386_PAGE_ENTRY_CACHE_DISABLE = (1 << 4),
   I386_PAGE_ENTRY_ACCESSED      = (1 << 5),
   I386_PAGE_ENTRY_DIRTY         = (1 << 6),
   I386_PAGE_ENTRY_GRANULARITY   = (1 << 7),
   I386_PAGE_ENTRY_GLOBAL        = (1 << 8),
   I386_PAGE_ENTRY_RESERVED_0    = (1 << 9),
   I386_PAGE_ENTRY_RESERVED_1    = (1 << 10),
   I386_PAGE_ENTRY_RESERVED_2    = (1 << 11)
} i386PageEntryAttributes;

typedef uint32 i386PageEntry;

typedef struct
{
   i386PageEntry entries [I386_PAGE_DIRECTORY_ENTRIES];
} i386PageDirectory;

typedef struct
{
   i386PageEntry entries [I386_PAGE_TABLE_ENTRIES];
} i386PageTable;

#endif /* __I386_PAGE_H__ */

