#ifndef __IDE_DRIVE_H__
#define __IDE_DRIVE_H__

#include <ide/ide.h>

struct IDEController;

typedef enum
{
        IDR_DRIVE_NUMBER_0 = 0,
        IDE_DRIVE_NUMBER_1 = 1,
        IDE_DRIVE_COUNT    = 2
} IDEDriveNumber;

typedef struct IDEDrive
{
        Bool                   isAllocated;
        Bool                   isDetected;
        struct IDEController * controller; 
        IDEDriveNumber         number;
        IDEDriveIdentification identification;
        Size                   cylinders;
        Size                   heads;
        Size                   sectorsPerTrack;
        Size                   bufferSize;
        Bool                   has32BitIO;
} IDEDrive;

extern IDEDrive * ide_drive_detect    (struct IDEController *, IDEDriveNumber);
extern IDEDrive * ide_drive_allocate  (struct IDEController *, IDEDriveNumber);
extern void ide_drive_free            (IDEDrive *);

extern void ide_drive_write_buffer    (const IDEDrive *, const void *, Size);
extern void ide_drive_read_buffer     (const IDEDrive *, void *, Size);

extern IDEError ide_drive_read        (const IDEDrive *, uint16, uint8, uint8, void *, Size);
extern IDEError ide_drive_write       (const IDEDrive *, uint16, uint8, uint8, const void *, Size);

extern IDEError ide_drive_recalibrate (const IDEDrive *);
extern IDEError ide_drive_identify    (IDEDrive *);

extern char * ide_drive_identification_descramble (char *, Size);

#endif /* __IDE_DRIVE_H__ */

