#ifndef __INTERRUPT_INTERRUPT_H__
#define __INTERRUPT_INTERRUPT_H__

#include <utility/types.h>
#include <utility/list.h>
#include <timer/timer.h>

enum
{
        INTERRUPT_NUMBER_ANY = ~0
};

typedef uint32  InterruptNumber;
typedef SortKey InterruptHandlerPriority;

typedef void InterruptHandlerFunction (InterruptNumber, void *); 

typedef struct
{
        ListKeyNode                     handlerList;
        InterruptHandlerFunction *      handlerFunction;
        void *                          handlerData;
        InterruptHandlerPriority        priority;
        const char *                    name;
        InterruptNumber                 number;
} InterruptHandler;

extern Bool interrupt_handler_create  (InterruptHandler *,
                                       InterruptNumber,
                                       InterruptHandlerFunction *,
                                       void *,
                                       InterruptHandlerPriority,
                                       const char *);
extern void interrupt_handler_destroy (InterruptHandler *);

extern void interrupt_enable          (InterruptNumber);
extern void interrupt_disable         (InterruptNumber);

extern Bool interrupt_wait            (InterruptNumber, TimeOffset *);

extern Size interrupt_level           (void);

#endif /* __INTERRUPT_INTERRUPT_H__ */

