#ifndef __INTERRUPT_IRQ_H__
#define __INTERRUPT_IRQ_H__

#include <utility/types.h>
#include <utility/list.h>

typedef enum
{
        IRQ_BASE_EXCEPTIONS = 0,
        IRQ_BASE_INTERRUPTS = 0x20,
        IRQ_BASE_SOFTWARE   = 0x30
} IRQBase;

enum
{
        IRQ_COUNT = 48
};

enum
{
        IRQ_STUB_CODE_SIZE = 16
};

typedef struct
{
        uint8   code [IRQ_STUB_CODE_SIZE];
} IRQStub;

typedef uint32  IRQNumber;
typedef uint32  IRQParameter;
typedef SortKey IRQHandlerPriority;

typedef struct
{
        uint32          EAX;
        uint32          ECX;
        uint32          EDX;
        uint32          EBX;
        uint32          ESP;
        uint32          EBP;
        uint32          ESI;
        uint32          EDI;
        uint32          irqVector;
        uint32          errorCode;
        uint32          EIP;
        uint32          CS;
        uint32          EFLAGS;
} IRQRegisters;

typedef struct
{
        ListKeyNode        handlerList;
        void *      	   handlerFunction;
        void *             handlerData;
        IRQHandlerPriority priority;
        const char *       name;
	IRQNumber	   number;
} IRQHandler;

extern IRQStub  irq_stubs [];

extern void     interrupt_dispatch (IRQRegisters);
extern void     exception_dispatch (IRQRegisters);

extern void     irq_initialize     (void);

extern void	irq_handler_insert (IRQHandler *);
extern void	irq_handler_remove (IRQHandler *);

#endif /* __INTERRUPT_IRQ_H__ */

