#include <interrupt/pic.h>

static uint8 picMasks [2] = { ~ (1 << 2), 0xFF };

void
pic_initialize (void)
{
        /* Disable all IRQs temporarily */

        pic_write_register (PIC_IO_BASE_0, 
                            PIC_REGISTER_INTERRUPT_ENABLE, 
                            0xFF);
        pic_write_register (PIC_IO_BASE_1, 
                            PIC_REGISTER_INTERRUPT_ENABLE, 
                            0xFF);

        pic_write_register (PIC_IO_BASE_0,
                            PIC_REGISTER_COMMAND,
                            PIC_COMMAND_INITIALIZATION_MODE);
        pic_write_register (PIC_IO_BASE_1,
                            PIC_REGISTER_COMMAND,
                            PIC_COMMAND_INITIALIZATION_MODE);

        /* Remap IRQs 0-7 to 0x20-0x27 and 8-15 to 0x28-0x30 */

        pic_write_register (PIC_IO_BASE_0, PIC_REGISTER_INITIALIZE, 0x20);
        pic_write_register (PIC_IO_BASE_1, PIC_REGISTER_INITIALIZE, 0x28);

        /* Slave 0 attached to IRQ 2, Slave 1 attached to IRQ 1 */

        pic_write_register (PIC_IO_BASE_0, PIC_REGISTER_INITIALIZE, (1 << 2));
        pic_write_register (PIC_IO_BASE_1, PIC_REGISTER_INITIALIZE, (1 << 1));

        pic_write_register (PIC_IO_BASE_0, PIC_REGISTER_INITIALIZE, 0x01);
        pic_write_register (PIC_IO_BASE_1, PIC_REGISTER_INITIALIZE, 0x01);

        /* Initialize IRQ masks to the defaults */

        pic_write_register (PIC_IO_BASE_0, 
                            PIC_REGISTER_INTERRUPT_ENABLE, 
                            picMasks [0]);
        pic_write_register (PIC_IO_BASE_1, 
                            PIC_REGISTER_INTERRUPT_ENABLE, 
                            picMasks [1]);
}

void
pic_acknowledge_irq (uint8 irq)
{
        pic_write_register ((irq < 8) ? PIC_IO_BASE_0 : PIC_IO_BASE_1,
                            PIC_REGISTER_COMMAND,
                            PIC_COMMAND_END_OF_INTERRUPT);
}

void
pic_mask_irq (uint8 irq)
{
        picMasks [irq / 8] |= (1 << (irq % 8));

        pic_write_register ((irq < 8) ? PIC_IO_BASE_0 : PIC_IO_BASE_1,
                            PIC_REGISTER_INTERRUPT_ENABLE,
                            picMasks [irq / 8]);
}

void
pic_unmask_irq (uint8 irq)
{
        picMasks [irq / 8] &= ~ (1 << (irq % 8));

        pic_write_register ((irq < 8) ? PIC_IO_BASE_0 : PIC_IO_BASE_1,
                            PIC_REGISTER_INTERRUPT_ENABLE,
                            picMasks [irq / 8]);
}


