#ifndef __INTERRUPT_PIC_H__
#define __INTERRUPT_PIC_H__

#include <i386/io.h>
#include <utility/inline.h>

typedef enum
{
        PIC_IO_BASE_0 = 0x20,
        PIC_IO_BASE_1 = 0xA0
} PICIOBase;

typedef enum
{
        PIC_REGISTER_DATA             = 0,
        PIC_REGISTER_COMMAND          = 0,
        PIC_REGISTER_INITIALIZE       = 1,
        PIC_REGISTER_INTERRUPT_ENABLE = 1
} PICRegister;

typedef enum
{
        PIC_COMMAND_READ_INTERRUPT_REQUESTS            = 0x0A,
        PIC_COMMAND_READ_INTERRUPTS_IN_SERVICE         = 0x0B,
        
        PIC_COMMAND_END_OF_INTERRUPT                   = 0x20,
        PIC_COMMAND_ROTATIN_PRIORITY_END_OF_INTERRUPT  = 0xA0,

        PIC_COMMAND_CLEAR_SPECIAL_MASK_MODE            = 0x48,
        PIC_COMMAND_SET_SPECIAL_MASK_MODE              = 0x68,
        
        PIC_COMMAND_END_OF_INTERRUPT_0                 = 0x60,
        PIC_COMMAND_END_OF_INTERRUPT_1                 = 0x61,
        PIC_COMMAND_END_OF_INTERRUPT_2                 = 0x62,
        PIC_COMMAND_END_OF_INTERRUPT_3                 = 0x63,
        PIC_COMMAND_END_OF_INTERRUPT_4                 = 0x64,
        PIC_COMMAND_END_OF_INTERRUPT_5                 = 0x65,
        PIC_COMMAND_END_OF_INTERRUPT_6                 = 0x66,
        PIC_COMMAND_END_OF_INTERRUPT_7                 = 0x67,
        
        PIC_COMMAND_SELECT_IRQ_PRIORITY_0              = 0xC0,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_1              = 0xC1,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_2              = 0xC2,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_3              = 0xC3,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_4              = 0xC4,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_5              = 0xC5,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_6              = 0xC6,
        PIC_COMMAND_SELECT_IRQ_PRIORITY_7              = 0xC7,

        PIC_COMMAND_INITIALIZATION_MODE                = 0x11
} PICCommand;        
       
INLINE uint8 pic_read_register (PICIOBase ioBase, PICRegister registerNumber)
{ return i386_input_8 (ioBase + registerNumber); }

INLINE void pic_write_register (PICIOBase ioBase, PICRegister registerNumber, uint8 data)
{ i386_output_8 (ioBase + registerNumber, data); }

extern void     pic_initialize          (void);
extern void     pic_acknowledge_irq     (uint8);
extern void     pic_mask_irq            (uint8);
extern void     pic_unmask_irq          (uint8);

#endif /* __INTERRUPT_PIC_H__ */

