#ifndef __KBC_KBC_H__
#define __KBC_KBC_H__

#include <i386/io.h>
#include <utility/inline.h>

typedef enum
{
   KBC_REGISTER_DATA    = 0x60,
   KBC_REGISTER_STATUS  = 0x64,
   KBC_REGISTER_COMMAND = 0x64
} KBCRegister;

typedef enum
{
   KBC_STATUS_OUTPUT_PENDING      = (1 << 0),
   KBC_STATUS_INPUT_PENDING       = (1 << 1),
   KBC_STATUS_SELF_TEST_SUCCEEDED = (1 << 2),
   KBC_STATUS_IS_COMMAND          = (1 << 3),
   KBC_STATUS_KEYBOARD_ENABLED    = (1 << 4),
   KBC_STATUS_AUX_PENDING         = (1 << 5),
   KBC_STATUS_COMMAND_TIMEOUT     = (1 << 6),
   KBC_STATUS_PARITY_ERROR        = (1 << 7)
} KBCStatus;

typedef enum
{
   KBC_DATA_TOO_MANY_KEY_PRESSES           = 0x00,
   KBC_DATA_SELF_TEST_SUCCEEDED            = 0x55,
   KBC_DATA_BASIC_ASSURANCE_TEST_SUCCEEDED = 0xAA,
   KBC_DATA_COMMAND_ECHO                   = 0xEE,
   KBC_DATA_COMMAND_ACKNOWLEDGE            = 0xFA,
   KBC_DATA_BASIC_ASSURANCE_TEST_FAILED    = 0xFC,
   KBC_DATA_RESEND_REQUEST                 = 0xFE,
   KBC_DATA_KEYBOARD_ERROR                 = 0xFF
} KBCData;

typedef enum
{
   KBC_COMMAND_SET_KEYBOARD_LEDS          = 0xED,
   KBC_COMMAND_ECHO                       = 0xEE,
   KBC_COMMAND_SELECT_SCANCODE_SET        = 0xF0,
   KBC_COMMAND_IDENTIFY_KEYBOARD          = 0xF2,
   KBC_COMMAND_TYPEMATIC_RATE_PROGRAMMING = 0xF3,
   KBC_COMMAND_ENABLE_KEYBOARD            = 0xF4,
   KBC_COMMAND_DISABLE_SCANNING           = 0xF5,
   KBC_COMMAND_ENABLE_SCANNING            = 0xF6,
   KBC_COMMAND_INTERNAL_DIAGNOSTICS       = 0xFF,

   KBC_COMMAND_READ_COMMAND_BYTE          = 0x20,
   KBC_COMMAND_WRITE_COMMAND_BYTE         = 0x60,
   KBC_COMMAND_ENABLE_AUX_INTERFACE       = 0xA7,
   KBC_COMMAND_DISABLE_AUX_INTERFACE      = 0xA8,
   KBC_COMMAND_TEST_AUX_INTERFACE         = 0xA9,
   KBC_COMMAND_SELF_TEST                  = 0xAA,
   KBC_COMMAND_TEST_KEYBOARD_INTERFACE    = 0xAB,
   KBC_COMMAND_DEACTIVATE_KEYBOARD        = 0xAD,
   KBC_COMMAND_ACTIVATE_KEYBOARD          = 0xAE,
   KBC_COMMAND_WRITE_AUX_INTERFACE        = 0xD4
} KBCCommand;

typedef enum
{
   KBC_COMMAND_BYTE_ENABLE_KEYBOARD_INTERRUPT = (1 << 0),
   KBC_COMMAND_BYTE_ENABLE_AUX_INTERRUPT      = (1 << 1),
   KBC_COMMAND_BYTE_SYSTEM_RESERVED_FLAG      = (1 << 2),
   KBC_COMMAND_BYTE_IGNORE_KEY_LOCK           = (1 << 3),
   KBC_COMMAND_BYTE_KEYBOARD_DISABLED         = (1 << 4),
   KBC_COMMAND_BYTE_AUX_DISABLED              = (1 << 5),
   KBC_COMMAND_BYTE_XT_SCANCODES              = (1 << 6)
} KBCCommandByte;

INLINE uint8 kbc_wait_for_write (void)
{
   uint8 status;

   do status = i386_input_8 (KBC_REGISTER_STATUS);
   while (status & KBC_STATUS_INPUT_PENDING);

   return status;
}

INLINE uint8 kbc_wait_for_read (void)
{
   uint8 status;
   
   do status = i386_input_8 (KBC_REGISTER_STATUS);
   while ((status & KBC_STATUS_OUTPUT_PENDING) != KBC_STATUS_OUTPUT_PENDING);

   return status;
}

INLINE uint8 kbc_read_status (void)
{ return i386_input_8 (KBC_REGISTER_STATUS); }

INLINE void kbc_write_command (uint8 command)
{ i386_output_8 (KBC_REGISTER_COMMAND, command); }

INLINE uint8 kbc_read_data (void)
{ return i386_input_8 (KBC_REGISTER_DATA); }

INLINE void kbc_write_data (uint8 data)
{ i386_output_8 (KBC_REGISTER_DATA, data); }

INLINE void kbc_clear_data (void)
{
   while (i386_input_8 (KBC_REGISTER_STATUS) & KBC_STATUS_OUTPUT_PENDING)
     i386_input_8 (KBC_REGISTER_DATA);
}

extern void  kbc_send_command           (uint8);
extern uint8 kbc_receive_data           (void);
extern Bool  kbc_acknowledge_data       (uint8);
extern Bool  kbc_reset                  (void);

extern void kbc_lock                    (void);
extern void kbc_unlock                  (void);

#endif /* __I386_KBC_H__ */
   
