#include <console/console.h>
#include <kbc/keyboard.h>
#include <utility/string.h>
#include <libc/limits.h>
#include <libc/ctype.h>
#include <libc/stdio.h>

FILE * stdin  = (FILE *) 1,
     * stdout = (FILE *) 2,
     * stderr = (FILE *) 3;

FILE *
fopen (const char * name, const char * mode)
{
   return NULL;
}

int
fclose (FILE * file)
{
   return -1;
}

int
feof (FILE * file)
{
   return 1;
}

int
fflush (FILE * file)
{
   return EOF;
}

size_t
fread (void * ptr, size_t size, size_t nmemb, FILE * file)
{
   uint8 * dstChar  = (uint8 *) ptr;
   size_t remaining = size * nmemb;
   int srcChar;

   while (remaining > 0)
   {
      srcChar = fgetc (file);

      if (srcChar == EOF)
        break;

      * dstChar = (uint8) srcChar;
      
      ++ dstChar;
      -- remaining;
   }
   
   return size * nmemb - remaining;
}

size_t
fwrite (const void * ptr, size_t size, size_t nmemb, FILE * file)
{
   char formatString [32];

   if (file != stdout &&
       file != stderr)
     return 0;

   string_format (formatString, sizeof (formatString), "%%.%ds", size);

   console_print_format (NULL, formatString, ptr);

   return size;
}

int
fputc (int c, FILE * file)
{
   if (file == stdout ||
       file == stderr)
   {  
      console_print_format (NULL, "%c", c);

      return c;
   }

   return EOF;
}

int
fputs (const char * string, FILE * file)
{
   return fprintf (file, string);
}

int
puts (const char * str)
{
   console_print_format (NULL, "%s\n", str);
   
   return 0;
}

int
printf (const char * format, ...)
{
   Arguments arguments;

   ARGUMENTS_BEGIN (arguments, format);

   console_print_arguments (NULL, format, arguments);

   ARGUMENTS_END (arguments);

   return 1;
}

int
fprintf (FILE * file, const char * format, ...)
{
   Arguments arguments;

   ARGUMENTS_BEGIN (arguments, format);

   if (file == stdout ||
       file == stderr)
     console_print_arguments (NULL, format, arguments);

   ARGUMENTS_END (arguments);

   return 1;
}

int
sprintf (char * buffer, const char * format, ...)
{
   int result;
   Arguments arguments;

   ARGUMENTS_BEGIN (arguments, format);

   result = (int) string_format_arguments (buffer, ~0, format, arguments);

   ARGUMENTS_END (arguments);

   return result;
}

int
snprintf (char * buffer, size_t length, const char * format, ...)
{
   int result;
   Arguments arguments;

   ARGUMENTS_BEGIN (arguments, format);

   result = (int) string_format_arguments (buffer, length, format, arguments);

   ARGUMENTS_END (arguments);

   return result;
}

int
vprintf (const char * format, va_list ap)
{
   console_print_arguments (NULL, format, ap);

   return 1;
}

int
vfprintf (FILE * file, const char * format, va_list ap)
{
  if (file == stdout ||
      file == stderr)
  {
     console_print_arguments (NULL, format, ap);

     return 1;
  }

  return 0;
}

int
vsprintf (char * buffer, const char * format, va_list ap)
{
   return (int) string_format_arguments (buffer, ~0, format, ap);
}

int
vsnprintf (char * buffer, size_t length, const char * format, va_list ap)
{
   return (int) string_format_arguments (buffer, length, format, ap);
}

int
sscanf (const char * source, const char * format, ...)
{
   return 0;
}

int
fgetc (FILE * file)
{
   KeyEvent event;

   if (file != stdin)
     return EOF;

   while (keyboard_read_events (NULL, & event, 1) == 1)
   {
      if (event.state == KEY_STATE_PRESSED &&
          isascii ((int) event.key) != 0)
      {
         console_print_format (NULL, "%c", event.key);
         
         return (int) event.key;
      }
   }

   return EOF;
}

char *
fgets (char * string, int size, FILE * file)
{
   uint8 * dstChar = (uint8 *) string;
   int srcChar;

   if (size <= 0)
     return NULL;

   while (size > 1)
   {
      srcChar = fgetc (file);

      if (srcChar == EOF)
      {
         if (dstChar == (uint8 *) string)
           return NULL;
         
         break;
      }
      
      * dstChar = (uint8) srcChar;
      
      ++ dstChar;
      -- size;

      if (srcChar == '\n')
        break;
   }
   
   * dstChar = '\0';
   
   return string;
}

char *
gets (char * string)
{
   return fgets (string, INT_MAX, stdin);
}

int
ungetc (int c, FILE * file)
{
   return EOF;
}

