#include <libc/libc.h>
#include <libc/stdlib.h>
#include <scheduler/thread.h>
#include <scheduler/semaphore.h>
#include <utility/string.h>
#include <utility/memory.h>
#include <memory/memory.h>
#include <linker/linker.h>

int
atoi (const char * string)
{
   return string_to_integer (string);
}

void *
calloc (size_t nmemb, size_t size)
{
   void * result = memory_allocate (nmemb * size);

   if (result != NULL)
     memory_zero (result, nmemb * size);

   return result;
}

void *
malloc (size_t size)
{
   return memory_allocate (size);
}

void *
realloc (void * memory, size_t size)
{
   return memory_resize (memory, size);
}

void
free (void * memory)
{
   memory_free (memory);
}

static unsigned randState = 1;

void
srand (unsigned newState)
{
   randState = newState;
}

int
rand (void)
{
   randState = (randState * 246897531) + 123456789;

   return (int) (randState & RAND_MAX);
}

char *
getenv (const char * variable)
{
   return NULL;
}

void
exit (int status)
{
    ProgramContext * context = (ProgramContext *) thread_get_data (NULL);

    if (context != NULL)
    {
       context -> exitStatus = status;

       semaphore_up (& context -> exitSemaphore);
    }

    thread_destroy (NULL);
}

void
abort (void)
{
    exit (EXIT_FAILURE);
}

static void
program_run (void * data)
{
    ProgramContext * context = (ProgramContext *) data;
   
    exit ((* context -> mainFunction) ((int) context -> argumentCount, context -> argumentVector));
}

int
system (const char * command)
{
    Size commandLength, argumentCount, argumentCurrent;
    char * commandLine, * commandCurrent;
    char ** argumentVector;
    int (* mainFunction) (int, char **);
    ProgramContext context;
    Export * export;

    if (command == NULL)
      return TRUE;

    commandLength = string_length (command) + 1;
    commandLine   = (char *) memory_allocate (commandLength);
    if (commandLine == NULL)
      return -1;
    
    memory_copy (commandLine, command, commandLength);

    for (argumentCount = 0,
	   commandCurrent = commandLine;
	 * commandCurrent != '\0';
	 ++ argumentCount)
    {
        while (* commandCurrent == ' ')
          ++ commandCurrent;

	if (* commandCurrent == '\0')
          break;

	do ++ commandCurrent;
	while (* commandCurrent != ' ' &&
	       * commandCurrent != '\0');
	 
	* commandCurrent = '\0';
	++ commandCurrent;
    }
  
    if (argumentCount > 0)
      argumentVector = (char **) memory_allocate (sizeof (char *) * argumentCount);
    else
      argumentVector = NULL;

    if (argumentVector == NULL)
    {
        memory_free (commandLine);
        
	return 127;
    }

    for (argumentCurrent = 0,
	   commandCurrent = commandLine;
	 argumentCurrent < argumentCount;
	 ++ argumentCurrent)
    {
	while (* commandCurrent == ' ')
           ++ commandCurrent;

	argumentVector [argumentCurrent] = commandCurrent;

	do ++ commandCurrent;
	while (commandCurrent [-1] != '\0');
    }

    export = linker_find_export (argumentVector [0], "main");

    if (export != NULL &&
        export -> type == EXPORT_TYPE_FUNCTION)
      mainFunction = (int (*) (int, char **)) (export -> section -> data + export -> offset);
    else
      mainFunction = NULL;
    
    context.argumentCount  = argumentCount;
    context.argumentVector = argumentVector;
    context.mainFunction   = mainFunction;
    context.exitStatus     = -1;

    semaphore_create (& context.exitSemaphore, 0);

    if (mainFunction != NULL &&
	thread_create (NULL, 
		       THREAD_PRIORITY_MINIMUM, 
		       THREAD_PRIORITY_MAXIMUM,
		       program_run,
		       & context,
		       NULL,
		       0) == TRUE)
      semaphore_down (& context.exitSemaphore);
    
    semaphore_destroy (& context.exitSemaphore);
    
    memory_free (argumentVector);
    memory_free (commandLine);

    return context.exitStatus;
}

    
