#ifndef __LINKER_PCOM_H__
#define __LINKER_PCOM_H__

#include <utility/types.h>
#include <utility/hash_table.h>
#include <linker/component.h>

static const char * const PCOM_SIGNATURE = "PCOM'01";

enum
{
        PCOM_SIGNATURE_LENGTH = 8
};

enum
{
        PCOM_VERSION = 1
};

typedef uint32  PComVersion;

enum
{
        PCOM_ADDRESS_NULL = 0
};

typedef uint32  PComAddress;

typedef uint32  PComSize;

enum
{
        PCOM_STRING_NULL = 0
};

typedef uint32  PComString;

enum
{
        PCOM_SECTION_NUMBER_NULL = ~0
};

typedef uint32  PComSectionNumber;

enum
{
        PCOM_SECTION_TYPE_STRINGS       = 1,
        PCOM_SECTION_TYPE_EXPORTS       = 2,
        PCOM_SECTION_TYPE_IMPORTS       = 3,
        PCOM_SECTION_TYPE_CONSTRUCTORS  = 4,
        PCOM_SECTION_TYPE_DESTRUCTORS   = 5
};

typedef uint32  PComSectionType;

typedef struct
{
        PComString name;
        PComString alias;
} PComSymbol;

typedef struct
{
        PComString      name;
        PComSectionType type;
        PComAddress     offset;
        PComSize        size;
        PComSize        symbolCount;
} PComSection;

typedef struct
{
        char                    signature [PCOM_SIGNATURE_LENGTH];
        PComVersion             version;
        PComString              name;
        PComSize                size;
        PComSize                sectionCount;
        PComSectionNumber       stringsSection;
        PComSectionNumber       constructorsSection;
        PComSectionNumber       destructorsSection;
} PComHeader;

typedef struct
{
        const char *    name;
        HashTable       symbols;
        PComHeader *    header;
        const char *    strings;
        uint8 *         data;
        Size            dataSize;
} PCom;

extern Bool             pcom_create             (PCom *, uint8 *, Size);
extern void             pcom_destroy            (PCom *);

extern Bool             pcom_find_symbol        (PCom *, 
                                                 const char *, 
                                                 PComSectionType *, 
                                                 const char **, 
                                                 const char **,
                                                 void **); 

extern Bool             pcom_validate           (PComHeader *);

extern PComSection *    pcom_find_section       (PCom *, PComSectionType, const char *);

extern Bool             pcom_read_symbols       (PCom *, PComSection *, void *);

extern Bool             pcom_add_imports        (PCom *, Component *);
extern Bool             pcom_add_exports        (PCom *, Component *);
extern Bool             pcom_add_constructors   (PCom *, Component *);
extern Bool             pcom_add_destructors    (PCom *, Component *);

#endif /* __LINKER_PCOM_H__ */

        
