#include <utility/memory.h>
#include <memory/memory.h>
#include <linker/component.h>
#include <linker/section.h>

Bool
section_create (Section ** result,
                Component * component,
                const char * name,
                SectionCapabilities capabilities,
                uint8 * data,
                Size dataAlignment,
                Size dataSize,
                Size relocationCount)
{
        Section * section = (Section *) memory_allocate (sizeof (Section));
        Export * export = NULL;

        if (section == NULL)
          return FALSE;

        section -> component    = component;
        section -> name         = name;
        section -> capabilities = capabilities;

        if (data == NULL &&
            dataSize > 0)
        {
           data = (uint8 *) memory_allocate (dataSize);
           if (data == NULL)
           {
              memory_free (section);

              return FALSE;
           }

           memory_zero (data, dataSize);

           section -> isAllocated = TRUE;
        }
        else
           section -> isAllocated = FALSE;

        section -> data          = data;
        section -> dataAlignment = dataAlignment;
        section -> dataSize      = dataSize;
        
        if (relocationCount > 0)
        {
           section -> relocations = (Relocation *) memory_allocate (relocationCount * sizeof (Relocation));
           if (section -> relocations == NULL)
           {
              if (section -> isAllocated == TRUE)
                memory_free (section -> data);

              memory_free (section);

              return FALSE;
           }

           memory_zero (section -> relocations, relocationCount * sizeof (Relocation));
        }
        else
           section -> relocations = NULL;

        section -> relocationCount = relocationCount;

        list_clear (& section -> exports);

        if (export_create (& export,
                           NULL,
                           section -> name,
                           section, 
                           EXPORT_TYPE_SECTION, 
                           0,
                           section -> dataSize) == FALSE ||
            import_create (NULL,
                           section -> component,
                           NULL,
                           section -> name,
                           IMPORT_TYPE_SECTION) == FALSE)
        {
           if (export != NULL)
             export_destroy (export);

           if (section -> relocations != NULL)
             memory_free (section -> relocations);

           if (section -> isAllocated == TRUE)
             memory_free (section -> data);

           memory_free (section);
           
           return FALSE;
        }

        list_insert_back (& component -> sections, section);

        if (result != NULL)
          * result = section;
        
        return TRUE;
}

void
section_destroy (Section * section)
{
        list_remove (& section -> sectionList);
        
        while (list_empty (& section -> exports) == FALSE)
          export_destroy ((Export *) list_front (& section -> exports));

        if (section -> relocationCount > 0)
          memory_free (section -> relocations);

        if (section -> isAllocated == TRUE)
          memory_free (section -> data);

        memory_free (section);
}

void
section_relocate (Section * section)
{
        Relocation * relocationCurrent = section -> relocations;

        for (relocationCurrent = section -> relocations;
             relocationCurrent < & section -> relocations [section -> relocationCount];
             ++ relocationCurrent)
          relocation_relocate (relocationCurrent, section);
}

