#ifndef __LINKER_SECTION_H__
#define __LINKER_SECTION_H__

#include <utility/list.h>
#include <linker/relocation.h>

struct Component;

typedef enum
{
        SECTION_CAPABILITY_READ    = (1 << 0),
        SECTION_CAPABILITY_WRITE   = (1 << 1),
        SECTION_CAPABILITY_EXECUTE = (1 << 2)
} SectionCapability;

typedef FlagSet SectionCapabilities;

typedef struct Section
{
        ListNode                sectionList;
        struct Component *      component;
        const char *            name;
        SectionCapabilities     capabilities;
        uint8 *                 data;
        Size                    dataAlignment;
        Size                    dataSize;
        Bool                    isAllocated;
        Relocation *            relocations;
        Size                    relocationCount;
        List                    exports;
} Section;

extern Bool     section_create         (Section **, 
                                        struct Component *, 
                                        const char *, 
                                        SectionCapabilities,
                                        uint8 *,
                                        Size,
                                        Size,
                                        Size);
extern void     section_destroy        (Section *);

extern void     section_relocate       (Section *);

#endif /* __LINKER_SECTION_H__ */

