#include <cache/slab.h>
#include <cache/cache.h>
#include <utility/string.h>
#include <memory/allocator.h>
#include <memory/memory.h>

static MemoryAllocator memoryAllocator;

Bool
__memory_enter__ (void)
{
   return memory_allocator_create (& memoryAllocator);
}

void
__memory_exit__ (void)
{
   memory_allocator_destroy (& memoryAllocator);
}

void *
memory_allocate_within (void * begin, void * end, Size size)
{
   return memory_allocator_allocate_within (& memoryAllocator, begin, end, size);
}

void *
memory_allocate (Size size)
{
   return memory_allocator_allocate (& memoryAllocator, size);
}

void *
memory_resize (void * memory, Size size)
{
   return memory_allocator_resize (& memoryAllocator, memory, size);
}

void
memory_claim (void * memory, Size size)
{
   memory_allocator_claim (& memoryAllocator, memory, size);
}

void
memory_release (void * memory, Size size)
{
   memory_allocator_release (& memoryAllocator, memory, size);
}

void
memory_free (void * memory)
{
   memory_allocator_free (& memoryAllocator, memory);
}

