#include <page/allocator.h>

static Page *
page_find (List * list,
           Page * begin,
	   Page * end)
{
   ListIterator page;

   for (page  = list_begin (list);
        page != list_end (list);
	page  = list_next (page))
   if (page >= & begin -> pageList &&
       page <= & end -> pageList)
     return (Page *) page;

   return NULL;
}

static Page *
page_break (PageAllocator * allocator,
            Page * page,
            PageAlignment requestedAlignment,
	    Page * begin,
	    Page * end)
{
     PageAlignment currentAlignment;
     Page        * bottomHalf,
                 * topHalf;

     currentAlignment = page -> alignment;

     do
     {
        bottomHalf = page;
	topHalf    = page + (1 << -- currentAlignment);

        if (end >= topHalf)
          page = topHalf;

        list_remove (& bottomHalf -> pageList);
     
        -- bottomHalf -> alignment;
        topHalf -> alignment = currentAlignment;
        topHalf -> address   = bottomHalf -> address + (1 << currentAlignment);
     
	list_insert_front (& allocator -> freeLists [currentAlignment],
	                   & bottomHalf -> pageList);
	list_insert_front (& allocator -> freeLists [currentAlignment],
	                   & topHalf -> pageList);
     } while (currentAlignment > requestedAlignment);
    
     return page;
}

static Page *
page_borrow (PageAllocator * allocator,
             PageAlignment currentAlignment,
	     PageAlignment requestedAlignment,
	     Page * begin,
	     Page * end)
{
     ListIterator   seeking;
     Page 	  * page,
          	  * boundary;
   
     while (currentAlignment > requestedAlignment &&
            currentAlignment < allocator -> freeListCount)
     {
        for (seeking  = list_begin (& allocator -> freeLists [currentAlignment]);
             seeking != list_end (& allocator -> freeLists [currentAlignment]);
             seeking  = list_next (seeking))
        {
           page = (Page *) seeking;

           if (page >= begin && 
	       page <= end)
             return page_break (allocator, page, requestedAlignment, begin, end);

           boundary = page + (1 << currentAlignment);

	   if (begin >= page &&
	       begin  < boundary)
	     return page_break (allocator, page, requestedAlignment, begin, end);
	
	   if (end >= page &&
	       end  < boundary)
	     return page_break (allocator, page, requestedAlignment, begin, end);
        }

        ++ currentAlignment;
     }

     return NULL;
}

Page *
page_allocator_allocate_within (PageAllocator * allocator,
                                PageAlignment alignment, 
		                PageAddress begin,
		                PageAddress end,
				void * owner)
{
     Page * result;

     if (begin % (1 << alignment))
     {
        begin -= begin % (1 << alignment);
	begin += (1 << alignment);
     }

     if (alignment >= allocator -> freeListCount ||
         begin > end)
       return NULL;

     result = page_find (& allocator -> freeLists [alignment],
                         allocator -> pages - allocator -> rangeBegin + begin,
			 allocator -> pages - allocator -> rangeBegin + end);
     if (result == NULL)
     {
        result = page_borrow (allocator, 
	                      alignment + 1, 
			      alignment,
			      allocator -> pages - allocator -> rangeBegin + begin,
			      allocator -> pages - allocator -> rangeBegin + end);
	if (result == NULL)
	  return NULL;
     }

     list_remove (& result -> pageList);

     if (owner == NULL)
       result -> owner = allocator;
     else
       result -> owner = owner;
     
     return result;
}

Page *
page_allocator_allocate (PageAllocator * allocator, 
		         PageAlignment alignment,
			 void * owner)
{ 
   return page_allocator_allocate_within (allocator, 
                                          alignment, 
                                          allocator -> rangeBegin, 
                                          allocator -> rangeEnd, 
                                          owner); 
}

