#include <utility/memory.h>
#include <page/allocator.h>

Size
page_allocator_size (PageAddress rangeBegin, PageAddress rangeEnd, Size alignmentMaximum)
{
   return sizeof (PageAllocator) + 
	  sizeof (Page) * (rangeEnd - rangeBegin) +
	  sizeof (List) * alignmentMaximum;
}

Bool
page_allocator_create (PageAllocator * allocator,
		       PageAddress rangeBegin,
		       PageAddress rangeEnd,
		       Size freeListCount,
		       PageAllocatorPriority priority,
		       PageAllocatorAttributes attributes)
{
	Page *		pages;
	Page *		pageLast;
	PageAddress	alignedBegin;
	PageAlignment	alignment;
	List *		freeLists;
	List *		freeListCurrent;
	Size		pageCount = rangeEnd - rangeBegin;

	alignedBegin = rangeBegin;
	if (alignedBegin % (1 << (freeListCount - 1)))
	{
	   alignedBegin -= alignedBegin % (1 << (freeListCount - 1));
           alignedBegin += (1 << (freeListCount - 1));
	}

	if (alignedBegin >= rangeEnd)
	  return FALSE;

	freeLists     = (List *) & allocator [1];
	pages         = (Page *) & freeLists [freeListCount];
	pageCount    -= (alignedBegin - rangeBegin);

	memory_zero (pages, sizeof (Page) * (rangeEnd - rangeBegin));

	memory_zero (& allocator -> allocatorList, sizeof (ListKeyNode));

	allocator -> allocatorList.key = priority;
	allocator -> attributes        = attributes;
	
	allocator -> pages	       = pages;
	allocator -> freeLists         = freeLists;
	allocator -> freeListCount     = freeListCount;
	allocator -> rangeBegin        = rangeBegin;
	allocator -> rangeEnd          = rangeEnd;

	pages += (alignedBegin - rangeBegin);

	for (freeListCurrent = & freeLists [freeListCount - 1],
	           alignment = freeListCount - 1;
	     freeListCurrent >= freeLists;
	     -- freeListCurrent,
	           -- alignment)
	{
	    pageLast   = pages + pageCount;
	    pageCount %= (1 << alignment);
	    pageLast  -= pageCount;
	    
	    list_clear (freeListCurrent);

	    list_insert_vector (list_end (freeListCurrent),
	                        pages,
				pageLast,
				sizeof (Page) * (1 << alignment));
	   
	    while (pages < pageLast)
	    {
	         pages -> alignment  = alignment;
                 pages -> address    = allocator -> rangeBegin + (pages - allocator -> pages);
		 pages              += (1 << alignment);
	    }
	}

	
	pages     = allocator -> pages + (alignedBegin - rangeBegin);
	pageCount = (alignedBegin - rangeBegin);

	for (freeListCurrent = & freeLists [freeListCount - 1],
	           alignment = freeListCount - 1;
	     freeListCurrent >= freeLists;
	     -- freeListCurrent,
	           -- alignment)
	{
	     pageLast   = pages - pageCount;
	     pageCount %= (1 << alignment);
	     pageLast  += pageCount;

	     list_insert_vector (list_end (freeListCurrent),
			         pageLast,
				 pages,
				 sizeof (Page) * (1 << alignment));

	     while (pages > pageLast)
	     {
		 pages              -= (1 << alignment);
		 pages -> alignment  = alignment;
		 pages -> address    = allocator -> rangeBegin + (pages - allocator -> pages);
             }
	}
	
	return TRUE;
}

void
page_allocator_destroy (PageAllocator * allocator)
{
   page_allocator_remove (allocator);
}

