#ifndef __PAGE_ALLOCATOR_H__
#define __PAGE_ALLOCATOR_H__

#include <utility/inline.h>
#include <utility/types.h>
#include <utility/list.h>
#include <i386/page.h>

typedef Size	PageAlignment;
typedef Address	PageAddress;

typedef SortKey	PageAllocatorPriority;
typedef FlagSet	PageAllocatorAttributes;

enum
{
	PAGE_ALLOCATOR_ATTRIBUTES_ANY = 0
};

typedef struct
{
	ListNode	pageList;
        unsigned        alignment : I386_PAGE_ALIGNMENT;
        unsigned        address : sizeof (PageAddress) * 8 - I386_PAGE_ALIGNMENT;
        void *          owner;
} Page;

typedef struct
{
	ListKeyNode		allocatorList;
	PageAllocatorAttributes	attributes;
	Page *			pages;
	PageAddress		rangeBegin;
	PageAddress		rangeEnd;
	List *			freeLists;
	Size			freeListCount;
} PageAllocator;

extern Size	page_allocator_size		(PageAddress, PageAddress, Size);

extern Bool	page_allocator_create		(PageAllocator *, 
						 PageAddress, 
						 PageAddress, 
						 Size, 
						 PageAllocatorPriority, 
						 PageAllocatorAttributes);

extern void	page_allocator_destroy  	(PageAllocator *);

extern Page *	page_allocator_allocate_within	(PageAllocator *, PageAlignment, PageAddress, PageAddress, void *);
extern Page *	page_allocator_allocate		(PageAllocator *, PageAlignment, void *);

extern Size	page_allocator_fill		(PageAllocator *, PageAlignment, Size);
extern Page *	page_allocator_coalesce		(PageAllocator *, Page *, PageAlignment);
extern void	page_allocator_free    		(PageAllocator *, Page *);

extern void	page_allocator_insert		(PageAllocator *);
extern void	page_allocator_remove		(PageAllocator *);

extern PageAllocator *	page_allocator_find_attributes	(PageAllocator *, PageAllocatorAttributes);
extern PageAllocator *	page_allocator_find_range	(PageAllocator *, PageAddress, PageAddress);
extern PageAllocator *	page_allocator_find_address	(PageAllocator *, PageAddress);

INLINE Page *	page_allocator_find		(PageAllocator * allocator, PageAddress address)
{ return & allocator -> pages [address - allocator -> rangeBegin]; }

#endif /* __PAGE_ALLOCATOR_H__ */

