#ifndef __PCI_DEVICE_H__
#define __PCI_DEVICE_H__

#include <pci/pci.h>
#include <utility/list.h>

typedef struct
{
	ListNode	deviceList;

	Bool		isAllocated;

	PCIBusNumber	busNumber;
	PCIDeviceNumber	deviceNumber;
	PCIVendorID	vendorID;
	PCIDeviceID	deviceID;
	PCIVendorID	subsystemVendorID;
	PCIDeviceID	subsystemDeviceID;
	PCIClassCode	classCode;
} PCIDevice;

extern void			pci_device_list			(List *);

extern const PCIDevice * 	pci_device_find			(const PCIDevice *, PCIVendorID, PCIDeviceID);
extern const PCIDevice *	pci_device_find_class		(const PCIDevice *, PCIClassCode);
extern const PCIDevice *	pci_device_find_subsystem	(const PCIDevice *, PCIVendorID, PCIDeviceID);

extern PCIDevice *		pci_device_allocate		(const PCIDevice *);
extern void			pci_device_free			(PCIDevice *);

extern Bool			pci_device_read_config_8	(const PCIDevice *, PCIRegisterNumber, uint8 *);
extern Bool			pci_device_read_config_16	(const PCIDevice *, PCIRegisterNumber, uint16 *);
extern Bool			pci_device_read_config_32	(const PCIDevice *, PCIRegisterNumber, uint32 *);
extern Bool			pci_device_write_config_8	(const PCIDevice *, PCIRegisterNumber, uint8);
extern Bool			pci_device_write_config_16	(const PCIDevice *, PCIRegisterNumber, uint16);
extern Bool			pci_device_write_config_32	(const PCIDevice *, PCIRegisterNumber, uint32);

#endif /* __PCI_DEVICE_H__ */

