#include <interrupt/interrupt.h>
#include <scheduler/thread.h>
#include <scheduler/critical.h>

void
critical_enter (void)
{
   Thread * thread = thread_current ();

   if (thread == NULL)
     return;

   if (thread -> criticalCount <= 0)
     interrupt_disable (INTERRUPT_NUMBER_ANY);

   ++ thread -> criticalCount;
}

void
critical_exit (void)
{
   Thread * thread = thread_current ();

   if (thread == NULL ||
       thread -> criticalCount <= 0)
     return;

   -- thread -> criticalCount;
   
   if (thread -> criticalCount <= 0)
     interrupt_enable (INTERRUPT_NUMBER_ANY);
}

void
critical_notify_enter (void)
{
   Thread * thread = thread_current ();
   
   ++ thread -> criticalCount;
}

void
critical_notify_exit (void)
{
   Thread * thread = thread_current ();
   
   if (thread -> criticalCount > 0)
     -- thread -> criticalCount;
}

Size
critical_count (void)
{
   Thread * thread = thread_current ();

   return thread -> criticalCount;
}

