%include "utility/assembly.i"
%include "i386/manage.i"
%include "i386/fpu.i"
%include "interrupt/irq.i"
%include "scheduler/registers.i"

section .text read nowrite exec contents

FUNCTION thread_registers_load
        push    esi
        push    edi

        mov     eax, cr0
        or      eax, I386_CR0_TASK_SWITCHED
        mov     cr0, eax

        mov     esi, [esp + 3 * 4]
        
        mov     edi, esp
        and     edi, ~(I386_STACK_INTERRUPT_LIMIT - 1)
        add     edi, I386_STACK_INTERRUPT_LIMIT - IRQRegisters.size

        mov     eax, [esi + ThreadRegisters.EIP]
        mov     ecx, [esi + ThreadRegisters.EFLAGS]

        mov     [edi + IRQRegisters.EIP], eax
        mov     [edi + IRQRegisters.EFLAGS], ecx

        add     esi, ThreadRegisters.EAX

        mov     ecx, 8
        rep     movsd

        pop     edi
        pop     esi
        ret
FUNCTION_END


FUNCTION thread_registers_save
        push    esi
        push    edi
      
        mov     esi, esp
        and     esi, ~(I386_STACK_INTERRUPT_LIMIT - 1)
        add     esi, I386_STACK_INTERRUPT_LIMIT - IRQRegisters.size

        mov     edi, [esp + 3 * 4]
        cmp     [edi + ThreadRegisters.fpuContext + i386FPUContext.state], dword I386_FPU_STATE_SAVE
        jne     .saveGeneralRegisters

        fnsave  [edi + ThreadRegisters.fpuContext]
        mov     [edi + ThreadRegisters.fpuContext + i386FPUContext.state], dword I386_FPU_STATE_RESTORE

.saveGeneralRegisters:
        mov     eax, [esi + IRQRegisters.EIP]
        mov     ecx, [esi + IRQRegisters.EFLAGS]

        mov     [edi + ThreadRegisters.EIP], eax
        mov     [edi + ThreadRegisters.EFLAGS], ecx

        add     edi, ThreadRegisters.EAX

        mov     ecx, 8
        rep     movsd

        pop     edi
        pop     esi
        ret
FUNCTION_END


FUNCTION thread_registers_switch
        pushf
        push    eax
        
        mov     eax, [esp + 3 * 4]
        test    eax, eax
        jz      .loadRegisters

        mov     [eax + ThreadRegisters.ECX], ecx
        mov     [eax + ThreadRegisters.EDX], edx
        mov     [eax + ThreadRegisters.EBX], ebx
        mov     [eax + ThreadRegisters.EBP], ebp
        mov     [eax + ThreadRegisters.ESI], esi
        mov     [eax + ThreadRegisters.EDI], edi

        mov     ecx, [esp]
        mov     edx, [esp + 4]
        mov     ebx, [esp + 2 * 4]
        lea     ebp, [esp + 3 * 4]

        mov     [eax + ThreadRegisters.EAX], ecx 
        mov     [eax + ThreadRegisters.EFLAGS], edx
        mov     [eax + ThreadRegisters.EIP], ebx 
        mov     [eax + ThreadRegisters.ESP], ebp

;       Disable full context switching for now... not needed.
;        mov     cx, es
;        mov     dx, cs
;        mov     bx, ss
;        mov     bp, ds
;        mov     si, fs
;        mov     di, gs

;        mov     [eax + ThreadRegisters.ES], ecx
;        mov     [eax + ThreadRegisters.CS], edx
;        mov     [eax + ThreadRegisters.SS], ebx
;        mov     [eax + ThreadRegisters.DS], ebp
;        mov     [eax + ThreadRegisters.FS], esi
;        mov     [eax + ThreadRegisters.GS], edi

;        mov     ecx, cr3
;        sldt    dword [eax + ThreadRegisters.LDTR]
;        mov     [eax + ThreadRegisters.CR3], ecx

        cmp     [eax + ThreadRegisters.fpuContext + i386FPUContext.state], dword I386_FPU_STATE_SAVE
        jne     .loadRegisters

        fnsave  [eax + ThreadRegisters.fpuContext]
        mov     [eax + ThreadRegisters.fpuContext + i386FPUContext.state], dword I386_FPU_STATE_RESTORE

.loadRegisters:
        mov     eax, cr0
        or      eax, I386_CR0_TASK_SWITCHED
        mov     cr0, eax

        mov     eax, [esp + 4 * 4]

;        mov     ecx, [eax + ThreadRegisters.ES]
;        mov     edx, [eax + ThreadRegisters.SS]
;        mov     ebx, [eax + ThreadRegisters.DS]
;        mov     ebp, [eax + ThreadRegisters.FS]
;        mov     esi, [eax + ThreadRegisters.GS]

;        mov     es, cx
;        mov     ss, dx
;        mov     ds, bx
;        mov     fs, bp
;        mov     gs, si

;        mov     ecx, [eax + ThreadRegisters.CS]
;        mov     [esp], .loadCS
;        mov     [esp + 4], ecx
;        jmp     far [esp]

;.loadCS:
;        mov     ecx, [eax + ThreadRegisters.CR3]
        mov     edx, [eax + ThreadRegisters.EIP]
        mov     ebx, [eax + ThreadRegisters.EFLAGS]
        mov     esp, [eax + ThreadRegisters.ESP]
        sub     esp, 8

;        lldt    [eax + ThreadRegisters.LDTR]
;        mov     cr3, ecx
        mov     [esp], ebx
        mov     [esp + 4], edx

        mov     ecx, [eax + ThreadRegisters.ECX]
        mov     edx, [eax + ThreadRegisters.EDX]
        mov     ebx, [eax + ThreadRegisters.EBX]
        mov     ebp, [eax + ThreadRegisters.EBP]
        mov     esi, [eax + ThreadRegisters.ESI]
        mov     edi, [eax + ThreadRegisters.EDI]
        mov     eax, [eax + ThreadRegisters.EAX]

        popf
        ret
FUNCTION_END


        extern thread_destroy

FUNCTION thread_entry
        push    eax
        call    ecx

	push	dword 0
	call	thread_destroy
	
        jmp	$
FUNCTION_END

