#include <i386/general.h>
#include <i386/manage.h>
#include <utility/memory.h>
#include <interrupt/exception.h>
#include <scheduler/thread.h>
#include <scheduler/registers.h>

static ExceptionHandler deviceNotAvailableHandler;

static void
device_not_available_handler (ExceptionNumber number,
                              ExceptionParameter parameter,
                              void * data)
{
        Thread * thread = thread_current ();

        i386_clear_task_switched ();

        if (thread -> registers.fpuContext.state == I386_FPU_STATE_INITIALIZE)
          i386_fpu_initialize ();
        else
          i386_fpu_restore (& thread -> registers.fpuContext);

        thread -> registers.fpuContext.state = I386_FPU_STATE_SAVE;
}
           
void
thread_registers_initialize (void)
{
        exception_handler_create (& deviceNotAvailableHandler,
                                  7,
                                  device_not_available_handler,
                                  NULL,
                                  0,
                                  "fpu");

        i386_set_CR0 (I386_CR0_MONITOR_COPROCESSOR | I386_CR0_NUMERIC_ERROR | i386_get_CR0 ());
}

void
thread_registers_create (ThreadRegisters * registers, Thread * thread)
{
   memory_zero (registers, sizeof (ThreadRegisters));

   registers -> EIP    = (uint32) thread_entry;
   registers -> EFLAGS = I386_EFLAGS_FLAG_RESERVED | I386_EFLAGS_FLAG_INTERRUPT;
   registers -> EAX    = (uint32) thread -> enterData;
   registers -> ECX    = (uint32) thread -> enterFunction;
   registers -> ESP    = (uint32) & thread -> stack [thread -> stackSize];

   registers -> ES = I386_SELECTOR_SYSTEM_DATA;
   registers -> CS = I386_SELECTOR_SYSTEM_CODE;
   registers -> SS = I386_SELECTOR_SYSTEM_DATA;
   registers -> DS = I386_SELECTOR_SYSTEM_DATA;
   registers -> FS = I386_SELECTOR_SYSTEM_DATA;
   registers -> GS = I386_SELECTOR_SYSTEM_DATA;

   registers -> fpuContext.state = I386_FPU_STATE_INITIALIZE;
}

void
thread_registers_destroy (ThreadRegisters * registers, Thread * thread)
{
}

