#ifndef __SCHEDULER_REGISTERS_H__
#define __SCHEDULER_REGISTERS_H__

#include <utility/types.h>
#include <i386/fpu.h>

struct Thread;

typedef struct
{
  uint32         CR3;
  uint32         EIP;
  uint32         EFLAGS;
  uint32         EAX;
  uint32         ECX;
  uint32         EDX;
  uint32         EBX;
  uint32         ESP;
  uint32         EBP;
  uint32         ESI;
  uint32         EDI;
  uint32         ES;
  uint32         CS;
  uint32         SS;
  uint32         DS;
  uint32         FS;
  uint32         GS;
  uint32         LDTR;
  i386FPUContext fpuContext;
} ThreadRegisters;

extern void     thread_registers_initialize (void);

extern void	thread_registers_create     (ThreadRegisters *, struct Thread *);
extern void     thread_registers_destroy    (ThreadRegisters *, struct Thread *);

extern void     thread_registers_load       (const ThreadRegisters *);
extern void     thread_registers_save       (ThreadRegisters *);
extern void     thread_registers_switch     (ThreadRegisters *, const ThreadRegisters *);

extern void     thread_entry                (void);

#endif /* __SCHEDULER_REGISTERS_H__ */

