#include <memory/memory.h>
#include <storage/type.h>
#include <storage/storage.h>
#include <storage/transfer_request.h>

Bool
transfer_request_create (TransferRequest ** result,
                         Storage * storage,
                         TransferRequestType type,
                         StorageBlock blockStart,
                         uint8 * buffer,
                         Size bufferSize,
                         TransferRequestSuccessFunction * successFunction,
                         void * successData,
                         TransferRequestFailureFunction * failureFunction,
                         void * failureData)
{
        TransferRequest * request = (TransferRequest *) memory_allocate (sizeof (TransferRequest));

        if (request == NULL)
          return FALSE;

        request -> storage         = storage;
        request -> type            = type;
        request -> blockStart      = blockStart;
        request -> buffer          = buffer;
        request -> bufferSize      = bufferSize;
        request -> successFunction = successFunction;
        request -> successData     = successData;
        request -> failureFunction = failureFunction;
        request -> failureData     = failureData;

        if (result != NULL)
          * result = request;

        mutex_lock (& storage -> lock);

        list_insert_back (& storage -> transferRequests, request);

        if (storage -> state == STORAGE_STATE_READY)
          storage_flush (storage);

        mutex_unlock (& storage -> lock);

        return TRUE;
}

void
transfer_request_destroy (TransferRequest * transferRequest)
{
        mutex_lock (& storage -> lock);

        list_remove (& transferRequest -> requestList);

        memory_free (transferRequest);

        mutex_unlock (& storage -> lock);
}

void
transfer_request_succeed (TransferRequest * transferRequest)
{
        Storage * storage = transferRequest -> storage;

        if (transferRequest -> successFunction != NULL)
          (* transferRequest -> successFunction) (storage,
                                                  transferRequest -> blockStart,
                                                  transferRequest -> buffer,
                                                  transferRequest -> bufferSize,
                                                  transferRequest -> successData);

        transfer_request_destroy (transferRequest);

        mutex_lock (& storage -> lock);
       
        storage -> state = STORAGE_STATE_READY;

        mutex_unlock (& storage -> lock);

        storage_flush (storage);
}

void
transfer_request_fail (TransferRequest * transferRequest)
{
        Storage * storage = transferRequest -> storage;

        if (transferRequest -> failureFunction != NULL)
          (* transferRequest -> failureFunction) (storage,
                                                  transferRequest -> blockStart,
                                                  transferRequest -> buffer,
                                                  transferRequest -> bufferSize,
                                                  transferRequest -> failureData);

        transfer_request_destroy (transferRequest);

        mutex_lock (& storage -> lock);

        storage -> state = STORAGE_STATE_READY;

        mutex_unlock (& storage -> lock);

        storage_flush (storage);
}

void
transfer_request_cancel (TransferRequest * transferRequest)
{
        Storage * storage = transferRequest -> storage;

        if (transferRequest -> failureFunction != NULL)
          (* transferRequest -> failureFunction) (storage,
                                                  transferRequest -> blockStart,
                                                  transferRequest -> buffer,
                                                  transferRequest -> bufferSize,
                                                  transferRequest -> failureData);

        transfer_request_destroy (transferRequest);
}

