#ifndef __UTILITY_HASH_TABLE_H__
#define __UTILITY_HASH_TABLE_H__

#include <utility/types.h>
#include <utility/list.h>

enum
{
    HASH_TABLE_EMPTY_THRESHOLD = 2,
    HASH_TABLE_GROWTH_SCALE    = 2,
    HASH_TABLE_GROWTH_OFFSET   = 1
};

typedef struct
{
    ListNode        bucketList;
    const char *    key;
    void *          values [1];
} HashTableNode;

typedef struct
{
    Size        nodeWidth;
    Size        nodeCount;
    Size        bucketCount;
    List *      buckets;
} HashTable;

extern Bool     hash_table_create       (HashTable *, Size);
extern void     hash_table_destroy      (HashTable *);

extern void **  hash_table_find         (HashTable *, const char *);
extern void **  hash_table_insert       (HashTable *, const char *);
extern Bool     hash_table_remove       (HashTable *, const char *);
extern void     hash_table_for_each     (HashTable *, Size, void (*) (void *));

extern Bool     hash_table_grow         (HashTable *, Size);
extern void     hash_table_clear        (HashTable *);

#endif /* __UTILITY_HASH_TABLE_H__ */

