#include <scheduler/mutex.h>
#include <utility/memory.h>
#include <vga/context.h>

static Mutex vgaMutex;
static VGAContext * contextCurrent = NULL;
static VGAContext contextDefault;
static VGAContext contextText80x25 =
{
   /* mode */ VGA_MODE_TEXT_80x25,
   /* videoMemory */ (void *) 0xB8000, 
   /* enterFunction */ NULL,
   /* enterData */ NULL,
   /* exitFunction */ NULL,
   /* exitData */ NULL,
   /* miscellaneousOutput */ 0x43,
   { /* crtRegisters */
     0x5F, 0x4F, 0x50, 0x82,
     0x55, 0x81, 0xBF, 0x1F,
     0x00, 0x4F, 0x00, 0x0F, 
     /*0x0E, 0xB0, 0x16, 0x30,*/
     0x00, 0x00, 0x00, 0x00,
     0x9C, 0x8E, 0x8F, 0x28,
     0x1F, 0x96, 0xB9, 0xA3,
     0xFF
   },
   { /* attributeRegisters */
     0x00, 0x01, 0x02, 0x03,
     0x04, 0x05, 0x06, 0x07,
     0x08, 0x09, 0x0A, 0x0B,
     0x0C, 0x0D, 0x0E, 0x0F,
     0x00, 0x00, 0x0F, 0x00,
     0x00
   },
   { /* graphicsRegisters */
     0x00, 0x00, 0x00, 0x00,
     0x00, 0x10, 0x0E, 0x00,
     0xFF
   },
   { /* sequencerRegisters */
     0x03, 0x01, 0x03, 0x00,
     0x02
   },
   { /* dacRegisters */
     0, 0, 0, /* black */
     0, 0, 32, /* blue */
     0, 32, 0, /* green */
     0, 32, 32, /* cyan */
     32, 0, 0, /* red */
     32, 0, 32, /* magenta */
     32, 16, 0, /* yellow */
     48, 48, 48, /* white */
     32, 32, 32, /* bold black */
     0, 0, 63, /* bold blue */
     0, 63, 0, /* bold green */
     0, 63, 63, /* bold cyan */
     63, 0, 0, /* bold red */
     63, 0, 63, /* bold magenta */
     63, 63, 0, /* bold yellow */
     63, 63, 63 /* bold white */
   }
};

static VGAContext contextImage640x480x4 = 
{
   /* mode */ VGA_MODE_IMAGE_640x480x4,
   /* videoMemory */ (void *) 0xA0000,
   /* enterFunction */ NULL,
   /* enterData */ NULL,
   /* exitFunction */ NULL,
   /* exitData */ NULL,
   /* miscellaneousOutput */ 0xE3,
   { /* crtRegisters */
     0x5F, 0x4F, 0x50, 0x82,
     0x54, 0x80, 0x0B, 0x3E,
     0x00, 0x40, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00,
     0xEA, 0x8C, 0xDF, 0x28,
     0x00, 0xE7, 0x04, 0xE3,
     0xFF
   },
   { /* attributeRegisters */
     0x00, 0x01, 0x02, 0x03,
     0x04, 0x05, 0x06, 0x07,
     0x08, 0x09, 0x0A, 0x0B,
     0x0C, 0x0D, 0x0E, 0x0F,
     0x01, 0x00, 0x0F, 0x00,
     0x00
   },
   { /* graphicsRegisters */
     0x00, 0x00, 0x00, 0x20,
     0x00, 0x00, 0x05, 0x0F,
     0xFF
   },
   { /* sequencerRegisters */
     0x03, 0x01, 0x0F, 0x00,
     0x06
   },
   { /* dacRegisters */
     0, 0, 0, /* black */
     0, 0, 32, /* blue */
     0, 32, 0, /* green */
     0, 32, 32, /* cyan */
     32, 0, 0, /* red */
     32, 0, 32, /* magenta */
     32, 16, 0, /* yellow */
     48, 48, 48, /* white */
     32, 32, 32, /* bold black */
     0, 0, 63, /* bold blue */
     0, 63, 0, /* bold green */
     0, 63, 63, /* bold cyan */
     63, 0, 0, /* bold red */
     63, 0, 63, /* bold magenta */
     63, 63, 0, /* bold yellow */
     63, 63, 63 /* bold white */
   }
};

static VGAContext contextImage320x200x8 =
{
   /* mode */ VGA_MODE_IMAGE_320x200x8,
   /* videoMemory */ (void *) 0xA0000,
   /* enterFunction */ NULL,
   /* enterData */ NULL,
   /* exitFunction */ NULL,
   /* exitData */ NULL,
   /* miscellaneousOutput */ 0x63,
   { /* crtRegisters */
     0x5F, 0x4F, 0x50, 0x82,
     0x54, 0x80, 0xBF, 0x1F,
     0x00, 0x41, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00,
     0x9C, 0x8E, 0x8F, 0x28,
     0x40, 0x96, 0xB9, 0xA3,
     0xFF
   },
   { /* attributeRegisters */
     0x00, 0x01, 0x02, 0x03,
     0x04, 0x05, 0x06, 0x07,
     0x08, 0x09, 0x0A, 0x0B,
     0x0C, 0x0D, 0x0E, 0x0F,
     0x41, 0x00, 0x0F, 0x00,
     0x00
   },
   { /* graphicsRegisters */
     0x00, 0x00, 0x00, 0x00,
     0x00, 0x40, 0x05, 0x0F,
     0xFF
   },
   { /* sequencerRegisters */
     0x03, 0x01, 0x0F, 0x00,
     0x0E
   },
   { /* dacRegisters */
     0, 0, 0, /* black */
     0, 0, 32, /* blue */
     0, 32, 0, /* green */
     0, 32, 32, /* cyan */
     32, 0, 0, /* red */
     32, 0, 32, /* magenta */
     32, 16, 0, /* yellow */
     48, 48, 48, /* white */
     32, 32, 32, /* bold black */
     0, 0, 63, /* bold blue */
     0, 63, 0, /* bold green */
     0, 63, 63, /* bold cyan */
     63, 0, 0, /* bold red */
     63, 0, 63, /* bold magenta */
     63, 63, 0, /* bold yellow */
     63, 63, 63 /* bold white */
   }
};

Bool
__vga_enter__ (void)
{
    mutex_create (& vgaMutex);

    vga_context_create (& contextDefault,
                        VGA_MODE_TEXT_80x25,
                        NULL,
                        NULL,
                        NULL,
                        NULL);

    contextCurrent = & contextDefault;

    if (0) vga_context_activate (& contextDefault);

    return TRUE;
}

void
__vga_exit__ (void)
{
    mutex_destroy (& vgaMutex);
}

VGAContext *
vga_context_current (void)
{
     return contextCurrent;
}

Bool
vga_context_create (VGAContext * context,
                    VGAMode mode,
                    VGAContextEnterFunction * enterFunction,
                    void * enterData,
                    VGAContextExitFunction * exitFunction,
                    void * exitData)
{
    if (mode == VGA_MODE_TEXT_80x25)
      * context = contextText80x25;
    else
    if (mode == VGA_MODE_IMAGE_640x480x4)
      * context = contextImage640x480x4;
    else
    if (mode == VGA_MODE_IMAGE_320x200x8)
      * context = contextImage320x200x8;
    else
      return FALSE;

    context -> enterFunction = enterFunction;
    context -> enterData     = enterData;
    context -> exitFunction  = exitFunction;
    context -> exitData      = exitData;

    return TRUE;
}

void
vga_context_destroy (VGAContext * context)
{
     mutex_lock (& vgaMutex);

     if (context == contextCurrent)
       vga_context_activate (& contextDefault);

     mutex_unlock (& vgaMutex);
}

void *
vga_context_activate (VGAContext * context)
{
     Size index;
  
     if (context == contextCurrent)
     {
        if (context != NULL)
          return context -> videoMemory;
     
        return NULL;
     }

     mutex_lock (& vgaMutex);

     if (contextCurrent != NULL &&
         contextCurrent -> exitFunction != NULL)
       (* contextCurrent -> exitFunction) (contextCurrent, contextCurrent -> exitData);
     
     contextCurrent = context;
     if (context == NULL)
     {
        mutex_unlock (& vgaMutex);
        
        return NULL;
     }

     vga_output_8 (VGA_REGISTER_MISCELLANEOUS_OUTPUT_WRITE,
                   context -> miscellaneousOutput |
                   VGA_FLAG_MISCELLANEOUS_OUTPUT_COLOR_SELECT);

     vga_sequencer_output (VGA_INDEX_SEQUENCER_RESET, VGA_FLAG_SEQUENCER_RESET_ASYNCHRONOUS_RESET_DISABLE);
     vga_sequencer_output (VGA_INDEX_SEQUENCER_CLOCKING_MODE, 
                           context -> sequencerRegisters [VGA_INDEX_SEQUENCER_CLOCKING_MODE] | 
                           VGA_FLAG_SEQUENCER_CLOCKING_MODE_SCREEN_DISABLE);
     
     for (index = 2;
          index < VGA_INDEX_SEQUENCER_COUNT;
          ++ index)
       vga_sequencer_output (index, context -> sequencerRegisters [index]);
    
     vga_sequencer_output (VGA_INDEX_SEQUENCER_RESET,
                           VGA_FLAG_SEQUENCER_RESET_SYNCHRONOUS_RESET_DISABLE |
                           VGA_FLAG_SEQUENCER_RESET_ASYNCHRONOUS_RESET_DISABLE);

     vga_crt_output (VGA_INDEX_CRT_VERTICAL_RETRACE_END,
                     context -> crtRegisters [VGA_INDEX_CRT_VERTICAL_RETRACE_END] & ~ VGA_FLAG_CRT_VERTICAL_RETRACE_END_PROTECT_REGISTERS);
     
     for (index = 0;
          index < VGA_INDEX_CRT_COUNT;
          ++ index)
       vga_crt_output (index, context -> crtRegisters [index]);

     for (index = 0;
          index < VGA_INDEX_GRAPHICS_COUNT;
          ++ index)
       vga_graphics_output (index, context -> graphicsRegisters [index]);

     for (index = 0;
          index < VGA_INDEX_ATTRIBUTE_COUNT;
          ++ index)
     {
        (void) vga_input_8 (VGA_REGISTER_INPUT_STATUS_1_COLOR);
        vga_attribute_output (index, context -> attributeRegisters [index]);
     }
     
     vga_output_8 (VGA_REGISTER_DAC_INDEX_WRITE, 0);

     for (index = 0;
          index < 3 * VGA_INDEX_DAC_COUNT;
          ++ index)
       vga_output_8 (VGA_REGISTER_DAC_DATA, context -> dacRegisters [index]);

     vga_sequencer_output (VGA_INDEX_SEQUENCER_CLOCKING_MODE,
                           context -> sequencerRegisters [VGA_INDEX_SEQUENCER_CLOCKING_MODE]);
    
     (void) vga_input_8 (VGA_REGISTER_INPUT_STATUS_1_COLOR);
     vga_output_8 (VGA_REGISTER_ATTRIBUTE_DATA_WRITE, VGA_FLAG_ATTRIBUTE_PALETTE_ADDRESS_SOURCE);
     
     if (context -> enterFunction != NULL)
       (* context -> enterFunction) (context, context -> enterData); 

     mutex_unlock (& vgaMutex);

     return context -> videoMemory;
}

void
vga_context_set_color (VGAContext * context, uint8 index, Size colorCount, const uint8 * colors)
{
     memory_copy (& context -> dacRegisters [3 * index], colors, 3 * colorCount);

     mutex_lock (& vgaMutex);
     
     if (context == contextCurrent)
     {
        vga_output_8 (VGA_REGISTER_DAC_INDEX_WRITE, index);

        for (colorCount *= 3;
             colorCount > 0;
             -- colorCount,
                ++ colors)
          vga_output_8 (VGA_REGISTER_DAC_DATA, * colors); 
     }

     mutex_unlock (& vgaMutex);
}

void
vga_context_get_color (VGAContext * context, uint8 index, Size colorCount, uint8 * colors)
{
     memory_copy (colors, & context -> dacRegisters [3 * index], 3 * colorCount);
}

void
vga_context_set_logic (VGAContext * context, VGALogic logic, uint8 mask)
{
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_DATA_ROTATE] = logic & 0x0F;
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_BIT_MASK]    = mask;

     mutex_lock (& vgaMutex);
     
     if (context == contextCurrent)
     {
        vga_graphics_output (VGA_INDEX_GRAPHICS_DATA_ROTATE, logic & 0x0F);
        vga_graphics_output (VGA_INDEX_GRAPHICS_BIT_MASK, mask);
     }

     mutex_unlock (& vgaMutex);
}

void
vga_context_get_logic (VGAContext * context, VGALogic * logic, uint8 * mask)
{
     * logic = context -> graphicsRegisters [VGA_INDEX_GRAPHICS_DATA_ROTATE] & 0x0F;
     * mask  = context -> graphicsRegisters [VGA_INDEX_GRAPHICS_BIT_MASK];
}

void
vga_context_set_blinking (VGAContext * context, Bool enabled)
{
     uint8 modeControl;

     modeControl = context -> attributeRegisters [VGA_INDEX_ATTRIBUTE_MODE_CONTROL];

     if (enabled == TRUE)
       modeControl |= VGA_FLAG_ATTRIBUTE_BLINK_ENABLE;
     else
       modeControl &= ~ VGA_FLAG_ATTRIBUTE_BLINK_ENABLE;

     context -> attributeRegisters [VGA_INDEX_ATTRIBUTE_MODE_CONTROL] = modeControl;

     mutex_lock (& vgaMutex);

     if (context == contextCurrent)
       vga_attribute_output (VGA_INDEX_ATTRIBUTE_MODE_CONTROL, modeControl);

     mutex_unlock (& vgaMutex);
}

Bool
vga_context_get_blinking (VGAContext * context)
{
     return (context -> attributeRegisters [VGA_INDEX_ATTRIBUTE_MODE_CONTROL] & VGA_FLAG_ATTRIBUTE_BLINK_ENABLE) != 0;
}

void
vga_context_set_override (VGAContext * context, uint8 value, uint8 mask)
{
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_SET_RESET_VALUE]  = value;
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_SET_RESET_ENABLE] = mask;

     mutex_lock (& vgaMutex);

     if (context == contextCurrent)
     {
        vga_graphics_output (VGA_INDEX_GRAPHICS_SET_RESET_VALUE, value);
        vga_graphics_output (VGA_INDEX_GRAPHICS_SET_RESET_ENABLE, mask);
     }

     mutex_unlock (& vgaMutex);
}

void
vga_context_get_override (VGAContext * context, uint8 * value, uint8 * mask)
{
     * value = context -> graphicsRegisters [VGA_INDEX_GRAPHICS_SET_RESET_VALUE];
     * mask  = context -> graphicsRegisters [VGA_INDEX_GRAPHICS_SET_RESET_ENABLE];
}

void
vga_context_set_write (VGAContext * context, VGAWrite writeMode, uint8 planes)
{
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_GRAPHICS_MODE] &= VGA_FLAG_GRAPHICS_GRAPHICS_MODE_WRITE_MODE_MASK;
     context -> graphicsRegisters [VGA_INDEX_GRAPHICS_GRAPHICS_MODE] |= writeMode & VGA_FLAG_GRAPHICS_GRAPHICS_MODE_WRITE_MODE_MASK;
     context -> sequencerRegisters [VGA_INDEX_SEQUENCER_MAP_MASK]     = planes & 0x0F;
    
     mutex_lock (& vgaMutex);
     
     if (context == contextCurrent)
     {
        vga_graphics_output (VGA_INDEX_GRAPHICS_GRAPHICS_MODE, 
                             context -> graphicsRegisters [VGA_INDEX_GRAPHICS_GRAPHICS_MODE]);
        vga_sequencer_output (VGA_INDEX_SEQUENCER_MAP_MASK, planes & 0x0F);
     }

     mutex_unlock (& vgaMutex);
}

void
vga_context_get_write (VGAContext * context, VGAWrite * writeMode, uint8 * planes)
{
     * writeMode = context -> graphicsRegisters [VGA_INDEX_GRAPHICS_GRAPHICS_MODE] & VGA_FLAG_GRAPHICS_GRAPHICS_MODE_WRITE_MODE_MASK;
     * planes    = context -> sequencerRegisters [VGA_INDEX_SEQUENCER_MAP_MASK] & 0x0F;
}

void
vga_context_set_cursor (VGAContext * context, Size x, Size y)
{
     Size address = context -> crtRegisters [VGA_INDEX_CRT_START_ADDRESS_LOW] + 
                    context -> crtRegisters [VGA_INDEX_CRT_START_ADDRESS_HIGH] * 256 + 
                    y * 80 + x;
     
     if (context -> mode != VGA_MODE_TEXT_80x25)
       return;
     
     context -> crtRegisters [VGA_INDEX_CRT_CURSOR_LOCATION_LOW]  = address & 0xFF;
     context -> crtRegisters [VGA_INDEX_CRT_CURSOR_LOCATION_HIGH] = (address / 256) & 0xFF;

     mutex_lock (& vgaMutex);

     if (context == contextCurrent)
     {
        vga_crt_output (VGA_INDEX_CRT_CURSOR_LOCATION_LOW, address & 0xFF);
        vga_crt_output (VGA_INDEX_CRT_CURSOR_LOCATION_HIGH, (address / 256) & 0xFF);
     }

     mutex_unlock (& vgaMutex);
}

void
vga_context_get_cursor (VGAContext * context, Size * x, Size * y)
{
     Size offset = context -> crtRegisters [VGA_INDEX_CRT_CURSOR_LOCATION_LOW] +
                   context -> crtRegisters [VGA_INDEX_CRT_CURSOR_LOCATION_HIGH] * 256 -
                   context -> crtRegisters [VGA_INDEX_CRT_START_ADDRESS_LOW] - 
                   context -> crtRegisters [VGA_INDEX_CRT_START_ADDRESS_HIGH];

     if (context -> mode != VGA_MODE_TEXT_80x25)
     {
        * y = 0;
        * x = 0;
        
        return;
     }
        
     * y = offset / 80;
     * x = offset % 80;
}

