#ifndef __VGA_CONTEXT_H__
#define __VGA_CONTEXT_H__

#include <utility/types.h>
#include <vga/io.h>

struct VGAContext;

typedef void VGAContextEnterFunction (struct VGAContext *, void *);
typedef void VGAContextExitFunction (struct VGAContext *, void *);

typedef enum
{
    VGA_MODE_TEXT_80x25      = 0,
    VGA_MODE_IMAGE_640x480x4 = 1,
    VGA_MODE_IMAGE_320x200x8 = 2
} VGAMode;

typedef enum
{
    VGA_TEXT_ATTRIBUTE_FOREGROUND_COLOR = (0xF << 0),
    VGA_TEXT_ATTRIBUTE_FONT_SELECT      = (1 << 3),
    VGA_TEXT_ATTRIBUTE_BACKGROUND_COLOR = (0xF << 4),
    VGA_TEXT_ATTRIBUTE_BLINK            = (1 << 7)
} VGATextAttribute;

typedef enum
{
    VGA_COLOR_BLACK        = 0,
    VGA_COLOR_BLUE         = 1,
    VGA_COLOR_GREEN        = 2,
    VGA_COLOR_CYAN         = 3,
    VGA_COLOR_RED          = 4,
    VGA_COLOR_MAGENTA      = 5,
    VGA_COLOR_YELLOW       = 6,
    VGA_COLOR_WHITE        = 7,
    
    VGA_COLOR_BOLD_BLACK   = 8,
    VGA_COLOR_BOLD_BLUE    = 9,
    VGA_COLOR_BOLD_GREEN   = 10,
    VGA_COLOR_BOLD_CYAN    = 11,
    VGA_COLOR_BOLD_RED     = 12,
    VGA_COLOR_BOLD_MAGENTA = 13,
    VGA_COLOR_BOLD_YELLOW  = 14,
    VGA_COLOR_BOLD_WHITE   = 15
} VGAColor;

typedef enum
{
    VGA_LOGIC_SET = 0,
    VGA_LOGIC_AND = 1,
    VGA_LOGIC_OR  = 2,
    VGA_LOGIC_XOR = 3
} VGALogic;

typedef enum
{
    VGA_WRITE_PLANAR      = 0,
    VGA_WRITE_INTERLEAVED = 2
} VGAWrite;

typedef struct VGAContext
{
    VGAMode                   mode;
    void  *                   videoMemory;
    VGAContextEnterFunction * enterFunction;
    void *                    enterData;
    VGAContextExitFunction *  exitFunction;
    void *                    exitData;
    uint8                     miscellaneousOutput;
    uint8                     crtRegisters [VGA_INDEX_CRT_COUNT];
    uint8                     attributeRegisters [VGA_INDEX_ATTRIBUTE_COUNT];
    uint8                     graphicsRegisters [VGA_INDEX_GRAPHICS_COUNT];
    uint8                     sequencerRegisters [VGA_INDEX_SEQUENCER_COUNT];
    uint8                     dacRegisters [3 * VGA_INDEX_DAC_COUNT];
} VGAContext;

extern Bool         vga_context_create       (VGAContext *, VGAMode, 
                                              VGAContextEnterFunction *, void *,
                                              VGAContextExitFunction *, void *);
extern void         vga_context_destroy      (VGAContext *);

extern void *       vga_context_activate     (VGAContext *);
extern VGAContext * vga_context_current      (void);

extern void         vga_context_set_color    (VGAContext *, uint8, Size, const uint8 *);
extern void         vga_context_get_color    (VGAContext *, uint8, Size, uint8 *);

extern void         vga_context_set_cursor   (VGAContext *, Size, Size);
extern void         vga_context_get_cursor   (VGAContext *, Size *, Size *);

extern void         vga_context_set_blinking (VGAContext *, Bool);
extern Bool         vga_context_get_blinking (VGAContext *);

extern void         vga_context_set_logic    (VGAContext *, VGALogic, uint8);
extern void         vga_context_get_logic    (VGAContext *, VGALogic *, uint8 *);

extern void         vga_context_set_override (VGAContext *, uint8, uint8);
extern void         vga_context_get_override (VGAContext *, uint8 *, uint8 *);

extern void         vga_context_set_write    (VGAContext *, VGAWrite, uint8);
extern void         vga_context_get_write    (VGAContext *, VGAWrite *, uint8 *);

#endif /* __VGA_CONTEXT_H__ */

