#ifndef __VGA_IO_H__
#define __VGA_IO_H__

#include <utility/types.h>
#include <utility/inline.h>
#include <i386/io.h>

typedef enum
{
    VGA_REGISTER_CRT_INDEX_MONO               = 0x3B4,
    VGA_REIGSTER_CRT_DATA_MONO                = 0x3B5,
    VGA_REGISTER_CRT_INDEX_COLOR              = 0x3D4,
    VGA_REGISTER_CRT_DATA_COLOR               = 0x3D5,
    
    VGA_REGISTER_ATTRIBUTE_INDEX              = 0x3C0,
    VGA_REGISTER_ATTRIBUTE_DATA_WRITE         = 0x3C0,
    VGA_REGISTER_ATTRIBUTE_DATA_READ          = 0x3C1,
    
    VGA_REGISTER_GRAPHICS_INDEX               = 0x3CE,
    VGA_REGISTER_GRAPHICS_DATA                = 0x3CF,
    
    VGA_REGISTER_SEQUENCER_INDEX              = 0x3C4,
    VGA_REGISTER_SEQUENCER_DATA               = 0x3C5,

    VGA_REGISTER_DAC_INDEX_READ               = 0x3C7,
    VGA_REGISTER_DAC_INDEX_WRITE              = 0x3C8,
    VGA_REGISTER_DAC_STATE                    = 0x3C7,
    VGA_REGISTER_DAC_DATA                     = 0x3C9,

    VGA_REGISTER_MISCELLANEOUS_OUTPUT_WRITE   = 0x3C2,
    VGA_REGISTER_MISCELLANEOUS_OUTPUT_READ    = 0x3CC,
    
    VGA_REGISTER_FEATURE_CONTROL_WRITE_MONO   = 0x3BA,
    VGA_REGISTER_FEATURE_CONTORL_WRITE_COLOR  = 0x3DA,
    VGA_REGISTER_FEATURE_CONTROL_READ         = 0x3CA,
    
    VGA_REGISTER_INPUT_STATUS_0               = 0x3C2,
    VGA_REGISTER_INPUT_STATUS_1_COLOR         = 0x3DA,
    VGA_REGISTER_INPUT_STATUS_1_MONO          = 0x3BA
} VGARegister;

typedef enum
{
    VGA_INDEX_CRT_HORIZONTAL_TOTAL                     = 0,
    VGA_INDEX_CRT_HORIZONTAL_DISPLAY_END               = 1,
    VGA_INDEX_CRT_HORIZONTAL_BLANKING_START            = 2,
    VGA_INDEX_CRT_HORIZONTAL_BLANKING_END              = 3,
    VGA_INDEX_CRT_HORIZONTAL_RETRACE_START             = 4,
    VGA_INDEX_CRT_HORIZONTAL_RETRACE_END               = 5,
    VGA_INDEX_CRT_VERTICAL_TOTAL                       = 6,
    VGA_INDEX_CRT_OVERFLOW                             = 7,
    VGA_INDEX_CRT_PRESET_ROW_SCAN                      = 8,
    VGA_INDEX_CRT_MAXIMUM_SCAN_LINE                    = 9,
    VGA_INDEX_CRT_CURSOR_START                         = 10,
    VGA_INDEX_CRT_CURSOR_END                           = 11,
    VGA_INDEX_CRT_START_ADDRESS_HIGH                   = 12,
    VGA_INDEX_CRT_START_ADDRESS_LOW                    = 13,
    VGA_INDEX_CRT_CURSOR_LOCATION_HIGH                 = 14,
    VGA_INDEX_CRT_CURSOR_LOCATION_LOW                  = 15,
    VGA_INDEX_CRT_VERTICAL_RETRACE_START               = 16,
    VGA_INDEX_CRT_VERTICAL_RETRACE_END                 = 17,
    VGA_INDEX_CRT_VERTICAL_DISPLAY_END                 = 18,
    VGA_INDEX_CRT_OFFSET                               = 19,
    VGA_INDEX_CRT_UNDERLINE_LOCATION                   = 20,
    VGA_INDEX_CRT_VERTICAL_BLANKING_START              = 21,
    VGA_INDEX_CRT_VERTICAL_BLANKING_END                = 22,
    VGA_INDEX_CRT_MODE_CONTROL                         = 23,
    VGA_INDEX_CRT_LINE_COMPARE                         = 24,
    VGA_INDEX_CRT_COUNT                                = 25,

    VGA_INDEX_ATTRIBUTE_PALETTE_0                      = 0,
    VGA_INDEX_ATTRIBUTE_PALETTE_1                      = 1,
    VGA_INDEX_ATTRIBUTE_PALETTE_2                      = 2,
    VGA_INDEX_ATTRIBUTE_PALETTE_3                      = 3,
    VGA_INDEX_ATTRIBUTE_PALETTE_4                      = 4,
    VGA_INDEX_ATTRIBUTE_PALETTE_5                      = 5,
    VGA_INDEX_ATTRIBUTE_PALETTE_6                      = 6,
    VGA_INDEX_ATTRIBUTE_PALETTE_7                      = 7,
    VGA_INDEX_ATTRIBUTE_PALETTE_8                      = 8,
    VGA_INDEX_ATTRIBUTE_PALETTE_9                      = 9,
    VGA_INDEX_ATTRIBUTE_PALETTE_10                     = 10,
    VGA_INDEX_ATTRIBUTE_PALETTE_11                     = 11,
    VGA_INDEX_ATTRIBUTE_PALETTE_12                     = 12,
    VGA_INDEX_ATTRIBUTE_PALETTE_13                     = 13,
    VGA_INDEX_ATTRIBUTE_PALETTE_14                     = 14,
    VGA_INDEX_ATTRIBUTE_PALETTE_15                     = 15,
    VGA_INDEX_ATTRIBUTE_MODE_CONTROL                   = 16,
    VGA_INDEX_ATTRIBUTE_OVERSCAN                       = 17,
    VGA_INDEX_ATTRIBUTE_COLOR_PLANE_ENABLE             = 18,
    VGA_INDEX_ATTRIBUTE_HORIZONTAL_PIXEL_PANNING       = 19,
    VGA_INDEX_ATTRIBUTE_COLOR_SELECT                   = 20,
    VGA_INDEX_ATTRIBUTE_COUNT                          = 21,
    
    VGA_INDEX_SEQUENCER_RESET                          = 0,
    VGA_INDEX_SEQUENCER_CLOCKING_MODE                  = 1,
    VGA_INDEX_SEQUENCER_MAP_MASK                       = 2,
    VGA_INDEX_SEQUENCER_CHARACTER_MAP_SELECT           = 3,
    VGA_INDEX_SEQUENCER_MEMORY_MODE                    = 4,
    VGA_INDEX_SEQUENCER_COUNT                          = 5,

    VGA_INDEX_GRAPHICS_SET_RESET_VALUE                 = 0,
    VGA_INDEX_GRAPHICS_SET_RESET_ENABLE                = 1,
    VGA_INDEX_GRAPHICS_COLOR_COMPARE                   = 2,
    VGA_INDEX_GRAPHICS_DATA_ROTATE                     = 3,
    VGA_INDEX_GRAPHICS_READ_MAP_SELECT                 = 4,
    VGA_INDEX_GRAPHICS_GRAPHICS_MODE                   = 5,
    VGA_INDEX_GRAPHICS_MISCELLANEOUS                   = 6,
    VGA_INDEX_GRAPHICS_COLOR_DO_NOT_CARE               = 7,
    VGA_INDEX_GRAPHICS_BIT_MASK                        = 8,
    VGA_INDEX_GRAPHICS_COUNT                           = 9,
    
    VGA_INDEX_DAC_PALETTE_0                            = 0,
    /* ... */
    VGA_INDEX_DAC_PALETTE_255                          = 255,
    VGA_INDEX_DAC_COUNT                                = 256
} VGAIndex;

typedef enum
{
    VGA_FLAG_SEQUENCER_RESET_SYNCHRONOUS_RESET_DISABLE  = (1 << 0),
    VGA_FLAG_SEQUENCER_RESET_ASYNCHRONOUS_RESET_DISABLE = (1 << 1),
    VGA_FLAG_SEQUENCER_CLOCKING_MODE_SCREEN_DISABLE     = (1 << 5),

    VGA_FLAG_ATTRIBUTE_PALETTE_ADDRESS_SOURCE           = (1 << 5),
    VGA_FLAG_ATTRIBUTE_BLINK_ENABLE                     = (1 << 3),
    
    VGA_FLAG_GRAPHICS_GRAPHICS_MODE_WRITE_MODE_MASK     = (3 << 0),

    VGA_FLAG_CRT_VERTICAL_RETRACE_END_PROTECT_REGISTERS = (1 << 7),

    VGA_FLAG_MISCELLANEOUS_OUTPUT_COLOR_SELECT          = (1 << 0),
    VGA_FLAG_MISCELLANEOUS_OUTPUT_RAM_ENABLE            = (1 << 1),
    VGA_FLAG_MISCELLANEOUS_OUTPUT_DOT_CLOCK_25_MHZ      = (0 << 2),
    VGA_FLAG_MISCELLANEOUS_OUTPUT_DOT_CLOCK_28_MHZ      = (1 << 2),
    VGA_FLAG_MISCELLANEOUS_OUTPUT_DOT_CLOCK_MASK        = (3 << 2),
    VGA_FLAG_MISCELLENAOUS_OUTPUT_PAGE_SELECT_LOW       = (0 << 4),
    VGA_FLAG_MISCELLANEOUS_OUTPUT_PAGE_SELECT_HIGH      = (1 << 4)
} VGAFlag;

INLINE uint8 vga_input_8 (uint16 port)
{ return i386_input_8 (port); }

INLINE void vga_output_8 (uint16 port, uint8 value)
{ i386_output_8 (port, value); }

INLINE void vga_output_16 (uint16 port, uint16 value)
{ i386_output_16 (port, value); }

INLINE uint8 vga_crt_input (VGAIndex index)
{
    vga_output_8 (VGA_REGISTER_CRT_INDEX_COLOR, index);
    
    return vga_input_8 (VGA_REGISTER_CRT_DATA_COLOR);
}

INLINE void vga_crt_output (VGAIndex index, uint8 value)
{
    vga_output_16 (VGA_REGISTER_CRT_INDEX_COLOR, index | (value << 8));
}

INLINE uint8 vga_sequencer_input (VGAIndex index)
{
    vga_output_8 (VGA_REGISTER_SEQUENCER_INDEX, index);

    return vga_input_8 (VGA_REGISTER_SEQUENCER_DATA);
}

INLINE void vga_sequencer_output (VGAIndex index, uint8 value)
{
    vga_output_16 (VGA_REGISTER_SEQUENCER_INDEX, index | (value << 8));
}

INLINE uint8 vga_graphics_input (VGAIndex index)
{
    vga_output_8 (VGA_REGISTER_GRAPHICS_INDEX, index);

    return vga_input_8 (VGA_REGISTER_GRAPHICS_DATA);
}

INLINE void vga_graphics_output (VGAIndex index, uint8 value)
{
    vga_output_16 (VGA_REGISTER_GRAPHICS_INDEX, index | (value << 8));
}

INLINE uint8 vga_attribute_input (VGAIndex index)
{
    vga_output_8 (VGA_REGISTER_ATTRIBUTE_INDEX, index);

    return vga_input_8 (VGA_REGISTER_ATTRIBUTE_DATA_READ);
}

INLINE void vga_attribute_output (VGAIndex index, uint8 value)
{
    vga_output_8 (VGA_REGISTER_ATTRIBUTE_INDEX, index);
    vga_output_8 (VGA_REGISTER_ATTRIBUTE_DATA_WRITE, value);
}

#endif /* __VGA_IO_H__ */

