#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <xcom/xcom.h>

int
string_find (FILE * file, XComSection * section, const char * name, xc_reference * result)
{
     char * strings = (char *) malloc (section -> size);
     xc_reference stringIndex;

     if (strings == NULL)
       return -1;

     if (fseek (file, section -> offset, SEEK_SET) != 0 ||
         fread (strings, section -> size, 1, file) != 1)
     {
         free (strings);

         return -1;
     }

     for (stringIndex = 0;
          stringIndex < section -> size;
          stringIndex += strlen (& strings [stringIndex]) + 1)
     {
         if (strcmp (& strings [stringIndex], name) == 0)
           break;
     }

     free (strings);
 
     if (stringIndex >= section -> size)
       return 1;
    
     * result = stringIndex;

     return 0;
}

int
main (int argc, char ** argv)
{
     FILE * file;
     XComHeader header;
     XComSection * sectionMap,
                 * currentSection;
     xc_reference sectionName,
                  sectionOffset;
     int argument;

     if (argc < 3)
       return EXIT_FAILURE;

     file = fopen (argv [1], "rb");
     if (file == NULL)
     {
        perror (argv [1]);
        
        return EXIT_FAILURE;
     }
     
     if (fread (& header, sizeof (XComHeader), 1, file) != 1 ||
         memcmp (header.signature, XCOM_SIGNATURE, XCOM_SIGNATURE_LENGTH) != 0 ||
         (header.referenceAttributes & XCOM_REFERENCE_ATTRIBUTE_SIZE) != XCOM_HOST_REFERENCE_ATTRIBUTE_SIZE ||
         (header.referenceAttributes & XCOM_REFERENCE_ATTRIBUTE_BIG_ENDIAN) != XCOM_HOST_REFERENCE_ATTRIBUTE_BIG_ENDIAN ||
         header.version != XCOM_VERSION)
     {
        fprintf (stderr, "%s: invalid XCOM header\n", argv [1]);
        
        fclose (file);

        return EXIT_FAILURE;
     }

     sectionMap = (XComSection *) malloc (header.sections * sizeof (XComSection));
     if (sectionMap == NULL ||
         fread (sectionMap, header.sections * sizeof (XComSection), 1, file) != 1)
     {
        perror (argv [1]);

        fclose (file);

        if (sectionMap != NULL)
          free (sectionMap);
        
        return EXIT_FAILURE;
     }

     for (currentSection = sectionMap;
          currentSection < & sectionMap [header.sections];
          ++ currentSection)
       if (currentSection -> type == XCOM_SECTION_TYPE_STRINGS)
         break;

     if (currentSection >= & sectionMap [header.sections] ||
         string_find (file, currentSection, argv [2], & sectionName) != 0)
     {
        fclose (file);

        free (sectionMap);
        
        return EXIT_FAILURE;
     }

     for (currentSection = sectionMap;
          currentSection < & sectionMap [header.sections];
          ++ currentSection)
     {
         if (currentSection -> nameIndex == sectionName)
           break;
     }
   
     fclose (file);

     free (sectionMap);

     if (currentSection >= & sectionMap [header.sections])
       return EXIT_FAILURE;
  
     sectionOffset = currentSection -> offset;

     for (argument = 3;
          argument < argc;
          ++ argument)
     {
         xc_reference offset = 0;

         sscanf (argv [argument], "%i", & offset);

         sectionOffset += offset;         
     }
   
     printf ("%.8x\n", sectionOffset);

     return EXIT_SUCCESS;
}

