#include <stdlib.h>
#include <stdio.h>
#include <linker/pcom.h>

Bool
pcom_dump (uint8 * memory, Size memorySize)
{
    PComHeader * pcomHeader = (PComHeader *) memory;
    PComSection * interfaces,
                * currentInterface;
    PComSymbol * symbols,
               * currentSymbol;
    const char * interfaceStrings;

    interfaces       = (PComSection *) & pcomHeader [1];
    interfaceStrings = (const char *) & memory [interfaces [pcomHeader -> stringsSection].offset];

    printf ("header: sectionCount = %d, strings = %d, constructors = %d, destructors = %d\n",
            pcomHeader -> sectionCount,
            pcomHeader -> stringsSection,
            pcomHeader -> constructorsSection,
            pcomHeader -> destructorsSection);
    printf ("component: %s\n", & interfaceStrings [pcomHeader -> name]);

    for (currentInterface = interfaces;
         currentInterface < & interfaces [pcomHeader -> sectionCount];
         ++ currentInterface)
    {
        printf ("\nsection: offset = %d, size = %d, symbolCount = %d",
                currentInterface -> offset,
                currentInterface -> size,
                currentInterface -> symbolCount);

        switch (currentInterface -> type)
        {
        case PCOM_SECTION_TYPE_IMPORTS:
          printf ("\nimports: %s\n", & interfaceStrings [currentInterface -> name]);
         
          break;

        case PCOM_SECTION_TYPE_EXPORTS:
          printf ("\nexports: %s\n", & interfaceStrings [currentInterface -> name]);

          break;
         
        case PCOM_SECTION_TYPE_CONSTRUCTORS:
          printf ("\nconstructors\n");

          break;

        case PCOM_SECTION_TYPE_DESTRUCTORS:
          printf ("\ndestructors\n");

          break;

        case PCOM_SECTION_TYPE_STRINGS:
          printf ("\nstrings\n");

          break;
        }
       
        if (currentInterface -> symbolCount > 0)
        {
           symbols = (PComSymbol *) & memory [currentInterface -> offset];
           
           for (currentSymbol = symbols;
                currentSymbol < & symbols [currentInterface -> symbolCount];
                ++ currentSymbol)
           {
               printf ("symbol: name = \"%s\", alias = \"%s\"\n",
                       & interfaceStrings [currentSymbol -> name],
                       & interfaceStrings [currentSymbol -> alias]);
           }
        }
        else
        {
           const char * currentChar;

           for (currentChar = interfaceStrings;
                currentChar < & interfaceStrings [currentInterface -> size];
                ++ currentChar)
             if (* currentChar >= ' ' && * currentChar <= '~')
               putchar (* currentChar);
             else
               putchar ('.');

           putchar ('\n');
        }
    }
    
    return TRUE;
}

int
main (int argc, char ** argv)
{
        FILE * file;
        uint8 * buffer = NULL;
        size_t length = 0;

        if (argc < 2)
          return EXIT_FAILURE;

        file = fopen (argv [1], "rb");
        if (file == NULL)
          return EXIT_FAILURE;

        if (fseek (file, 0, SEEK_END) != 0)
          return EXIT_FAILURE;

        length = (size_t) ftell (file);

        rewind (file);

        buffer = (uint8 *) malloc (length);
        if (buffer == NULL)
          return EXIT_FAILURE;

        if (fread (buffer, length, 1, file) != 1)
          return EXIT_FAILURE;

        fclose (file);

        if (pcom_dump (buffer, length) == FALSE)
          return EXIT_FAILURE;

        free (buffer);

        return EXIT_SUCCESS;
}

