#include <stdlib.h>
#include "error.h"
#include "pidl.h"

void
pidl_version (void)
{
    printf ("PIDL - POS IDL Compiler (%s) by Lee Salzman (eihrul@tunes.org)\n", __DATE__);
}

void
pidl_help (void)
{
    printf ("Usage: pidl [options] output component input1 ... inputN\n"
            "\n"
            "Options:\n"
            "   -h                      Display help information\n"
            "   -v                      Display version information\n"
            "   -m                      Display verbose messages\n"
            "   -i directory/           Search for input files in directory\n"
            "\n");
}

int
main (int argc, char ** argv)
{
    FILE * inputFile,
         * outputFile;
    int argumentCurrent,
        argumentCount;
    PIDL pidl;
    Definition * definition;

    for (argumentCurrent = 1,
           argumentCount = argc;
         argumentCurrent < argumentCount;
         ++ argumentCurrent)
    {
       if (argv [argumentCurrent] [0] == '-')
       {
           int switchArguments = 1;
           char * switchCurrent;
           
           for (switchCurrent = & argv [argumentCurrent] [1];
                * switchCurrent != '\0';
                ++ switchCurrent)
           switch (* switchCurrent)
           {
           case 'v':
               pidl_version ();

               return EXIT_SUCCESS;

           case 'm':
               pidl_verbose (1);

               break;

           case 'h':
               pidl_help ();

               return EXIT_SUCCESS;

           case 'i':
               ++ switchArguments;

               break;

           default:
               pidl_help ();

               return EXIT_FAILURE;
           }

           if (argumentCount - argumentCurrent < switchArguments)
           {
              pidl_help ();

              return EXIT_FAILURE;
           }

           while (switchArguments > 0)
           {
               char * switchArgument = argv [argumentCurrent];

               memcpy (& argv [argumentCurrent],
                       & argv [argumentCurrent + 1],
                       (argc - argumentCurrent - 1) * sizeof (char *));

               argv [argc - 1] = switchArgument;
 
               -- switchArguments;
               -- argumentCount;
           }
 
           -- argumentCurrent;
       }
    }

    if (argumentCount < 4)
    {
       pidl_help ();

       return EXIT_FAILURE;
    }

    if (pidl_create (& pidl) != 0)
      return EXIT_FAILURE;

    for (argumentCurrent = argumentCount;
         argumentCurrent < argc;
         ++ argumentCurrent)
    { 
       const char * switchCurrent;
       
       for (switchCurrent = & argv [argumentCurrent] [1];
            * switchCurrent != '\0';
            ++ switchCurrent)
       switch (* switchCurrent)
       { 
       case 'i':
           ++ argumentCurrent;
           
           pidl_message ("Adding search directory %s", argv [argumentCurrent]);

           if (pidl_add_directory (& pidl, argv [argumentCurrent]) != 0)
             pidl_error ("Failed to add search directory %s", argv [argumentCurrent]);

           break;

       default:
           break;
       }
    }

    for (argumentCurrent = 3;
         argumentCurrent < argumentCount;
         ++ argumentCurrent)
    {
       pidl_message ("Reading input file %s", argv [argumentCurrent]);

       inputFile = fopen (argv [argumentCurrent], "r");

       if (inputFile == NULL ||
           pidl_read (& pidl, inputFile, 0) != 0)
         pidl_error ("%s: Failed to read PIDL file", argv [argumentCurrent]);
      
       fclose (inputFile);
    }
  
    pidl_message ("Writing component %s to output file %s", argv [2], argv [1]);

    outputFile = fopen (argv [1], "wb");

    if (outputFile == NULL)
      pidl_error ("%s: Failed to create output file", argv [1]);

    definition = namespace_find (& pidl.namespace, argv [2]);
    if (definition == NULL ||
        definition -> component == NULL)
      pidl_error ("Can't find component %s", argv [2]);
   
    component_prune (definition -> component);

    if (pidl_write (& pidl, definition -> component, outputFile) != 0)
      pidl_error ("%s: Failed to write output file", argv [1]);

    fclose (outputFile);

    pidl_destroy (& pidl);

    return EXIT_SUCCESS;
}

   
