#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <xcom/xcom.h>

int
string_find (FILE * file, XComSection * section, const char * name, xc_reference * result)
{
     char * strings = (char *) malloc (section -> size);
     xc_reference stringIndex;

     if (strings == NULL)
       return -1;

     if (fseek (file, section -> offset, SEEK_SET) != 0 ||
         fread (strings, section -> size, 1, file) != 1)
     {
         free (strings);

         return -1;
     }

     for (stringIndex = 0;
          stringIndex < section -> size;
          stringIndex += strlen (& strings [stringIndex]) + 1)
     {
         if (strcmp (& strings [stringIndex], name) == 0)
           break;
     }

     free (strings);
 
     if (stringIndex >= section -> size)
       return 1;
    
     * result = stringIndex;

     return 0;
}

int
symbol_find (FILE * file, XComSection * section, xc_reference nameIndex, XComSymbol * result)
{
     XComSymbol * symbols = (XComSymbol *) malloc (section -> size);
     XComSymbol * currentSymbol,
                * endSymbol;

     if (symbols == NULL)
       return -1;

     if (fseek (file, section -> offset, SEEK_SET) != 0 ||
         fread (symbols, section -> size, 1, file) != 1)
     { 
       
         free (symbols);

         return -1;
     }

     for (currentSymbol = symbols,
              endSymbol = & symbols [section -> size / sizeof (XComSymbol)];
          currentSymbol < endSymbol;
          ++ currentSymbol)
     {
         if (currentSymbol -> nameIndex == nameIndex)
           break;
     }

     free (symbols);
 
     if (currentSymbol >= endSymbol)
       return 1;

     * result = * currentSymbol;

     return 0;
}

int
main (int argc, char ** argv)
{
     FILE * file;
     XComHeader header;
     XComSection * sectionMap,
                 * currentSection;
     XComSymbol symbol;
     xc_reference symbolName,
                  symbolValue;
     int argument;

     if (argc < 3)
       return EXIT_FAILURE;

     file = fopen (argv [1], "rb");
     if (file == NULL)
     {
        perror (argv [1]);

        return EXIT_FAILURE;
     }
     
     if (fread (& header, sizeof (XComHeader), 1, file) != 1 ||
         memcmp (header.signature, XCOM_SIGNATURE, XCOM_SIGNATURE_LENGTH) != 0 ||
         (header.referenceAttributes & XCOM_REFERENCE_ATTRIBUTE_SIZE) != XCOM_HOST_REFERENCE_ATTRIBUTE_SIZE ||
         (header.referenceAttributes & XCOM_REFERENCE_ATTRIBUTE_BIG_ENDIAN) != XCOM_HOST_REFERENCE_ATTRIBUTE_BIG_ENDIAN ||
         header.version != XCOM_VERSION)
     {
        fprintf (stderr, "%s: invalid XCOM header\n", argv [1]);
        
        fclose (file);

        return EXIT_FAILURE;
     }

     sectionMap = (XComSection *) malloc (header.sections * sizeof (XComSection));
     if (sectionMap == NULL ||
         fread (sectionMap, header.sections * sizeof (XComSection), 1, file) != 1)
     {
        perror (argv [1]);

        fclose (file);

        if (sectionMap != NULL)
          free (sectionMap);
        
        return EXIT_FAILURE;
     }

     for (currentSection = sectionMap;
          currentSection < & sectionMap [header.sections];
          ++ currentSection)
       if (currentSection -> type == XCOM_SECTION_TYPE_STRINGS)
         break;

     if (currentSection >= & sectionMap [header.sections] ||
         string_find (file, currentSection, argv [2], & symbolName) != 0)
     {
        fclose (file);

        free (sectionMap);
        
        return EXIT_FAILURE;
     }

     for (currentSection = sectionMap;
          currentSection < & sectionMap [header.sections];
          ++ currentSection)
     {
         if (currentSection -> nameIndex == symbolName)
         {
            symbol.type       = XCOM_SYMBOL_TYPE_SECTION;
            symbol.attributes = 0;
            symbol.size       = currentSection -> size;
            symbol.offset     = 0;
            symbol.nameIndex  = symbolName;

            break;
         }

         if (symbol_find (file, currentSection, symbolName, & symbol) == 0)
           break;
     }
   
     fclose (file);

     free (sectionMap);

     if (currentSection >= & sectionMap [header.sections])
       return EXIT_FAILURE;
   
     symbolValue = symbol.offset;

     if (symbol.type != XCOM_SYMBOL_TYPE_CONSTANT &&
         symbol.type != XCOM_SYMBOL_TYPE_FILE &&
         (symbol.attributes & XCOM_SYMBOL_ATTRIBUTE_EXTERNAL) == 0)
       symbolValue += sectionMap [currentSection -> dataIndex - 1].memoryRegion;

     for (argument = 3;
          argument < argc;
          ++ argument)
     {
         xc_reference offset = 0;

         sscanf (argv [argument], "%i", & offset);

         symbolValue += offset;         
     }
   
     printf ("%.8x\n", symbolValue);

     return EXIT_SUCCESS;
}

