#ifndef __XL_LIST_H__
#define __XL_LIST_H__

#include <stdlib.h>
#include "inline.h"

typedef struct ListNode
{
   struct ListNode *	next;
   struct ListNode *	previous;
} ListNode;

typedef ListNode * 		ListIterator;
typedef const ListNode *	ListConstIterator;

typedef struct
{
   ListNode	sentinel;
} List;

INLINE void 		list_clear	 	(List * list)
{ list -> sentinel.next = list -> sentinel.previous = & list -> sentinel; }

INLINE ListIterator	list_front		(const List * list)
{ return list -> sentinel.next; }

INLINE ListIterator	list_back		(const List * list)
{ return list -> sentinel.previous; }

INLINE ListIterator	list_begin		(const List * list)
{ return list -> sentinel.next; }

INLINE ListIterator	list_end		(const List * list)
{ return (ListIterator) & list -> sentinel; }

INLINE int		list_empty		(const List * list)
{ return list_begin (list) == list_end (list); }

INLINE ListIterator	list_next		(ListConstIterator position)
{ return position -> next; }

INLINE ListIterator	list_previous		(ListConstIterator position)
{ return position -> previous; }

INLINE ListIterator	list_insert		(ListIterator position, void * value)
{ 
  register ListNode * const node = (ListNode *) value;
  
  node -> next             = position;
  node -> previous         = position -> previous;
  
  position -> previous     = node;
  node -> previous -> next = node;
  
  return node;
}

INLINE ListIterator	list_insert_front	(List * list, void * value)
{ return list_insert (list_begin (list), value); }

INLINE ListIterator	list_insert_back	(List * list, void * value)
{ return list_insert (list_end (list), value); }

INLINE void *		list_remove		(ListIterator node)
{ 
  node -> previous -> next = node -> next;
  node -> next -> previous = node -> previous;

  return node;
}

INLINE void *		list_remove_front	(List * list)
{ return list_remove (list_front (list)); }

INLINE void *		list_remove_back	(List * list)
{ return list_remove (list_back (list)); }

INLINE ListIterator	list_move_to_front	(List * list, ListIterator iterator)
{ return list_insert_front (list, list_remove (iterator)); }

INLINE ListIterator	list_move_to_back	(List * list, ListIterator iterator)
{ return list_insert_back (list, list_remove (iterator)); }

INLINE ListIterator     list_splice             (List * list, ListIterator begin, ListIterator end)
{
    if (begin == end)
    {
       list_clear (list);

       return list_end (list);
    }

    list -> sentinel.next     = begin;
    list -> sentinel.previous = end -> previous;
    
    end -> previous           = begin -> previous;
    begin -> previous -> next = end;

    begin -> previous                 = & list -> sentinel;
    list -> sentinel.previous -> next = & list -> sentinel;

    return begin;
}

INLINE ListIterator	list_find_iterator	(const List * list, ListIterator iterator)
{
    ListIterator current;

    for (current = list_begin (list);
	 current != list_end (list) &&
	    current < iterator;
	 current = list_next (current));
    
    return current;
}

INLINE size_t           list_size               (const List * list)
{
    size_t size = 0;
    ListIterator current;

    for (current = list_begin (list);
         current != list_end (list);
         current = list_next (current))
      ++ size;

    return size;
}
                  
#endif /* __XL_LIST_H__ */

