#ifndef __XL_OBJECT_H__
#define __XL_OBJECT_H__

#include <stdlib.h>
#include <stdio.h>
#include "list.h"
#include "symbol.h"
#include "section.h"
#include "namespace.h"

typedef struct Object
{
        List            sections;
        List            importedSymbols;
        List            constantSymbols;
        List            sectionSymbols;
        Namespace       namespace;
} Object;

extern int              object_create           (Object *);
extern void             object_destroy          (Object *);

extern int              object_merge            (Object *, Object *);
extern int              object_anchor           (Object *, Address, const char *);
extern void             object_fixup            (Object *, Section *, size_t);
extern void             object_relocate         (Object *);
extern void             object_strip            (Object *, int);

extern Symbol *         object_find_symbol      (Object *, const char *);
extern Symbol *         object_define_symbol    (Object *, 
                                                 Section *, 
                                                 SymbolType, 
                                                 SymbolAttributes, 
                                                 const char *, 
                                                 Address, 
                                                 size_t); 
extern void             object_define_symbols   (Object *, Section *, List *);
extern Section *        object_find_section     (Object *, const char *);
extern Section *        object_define_section   (Object *, 
                                                 SectionType, 
                                                 SectionAttributes, 
                                                 SectionCapabilities, 
                                                 const char *, 
                                                 const char *, 
                                                 Address, 
                                                 size_t, 
                                                 size_t);
extern const char *     object_translate_string (Object *, const char *);

extern void             object_allocate_common_symbols (Object *, const char *);

extern int              object_read             (Object *, FILE *);
extern int              object_write            (Object *, FILE *, int);

extern int              elf_verify              (FILE *);
extern int              elf_read                (Object *, FILE *);
extern int              xcom_verify             (FILE *);
extern int              xcom_read               (Object *, FILE *);
extern int              xcom_write              (Object *, FILE *, int);

#endif /* __XL_OBJECT_H__ */

