#ifndef __XL_SECTION_H__
#define __XL_SECTION_H__

#include <stdlib.h>
#include <stdio.h>
#include "types.h"
#include "list.h"

struct Object;

typedef enum
{
        SECTION_TYPE_TEMPORARY   = 0,
        SECTION_TYPE_INTERFACE   = 2,
        SECTION_TYPE_RESOURCE    = 3,
        SECTION_TYPE_PROGRAM     = 4
} SectionType;

typedef enum
{
        SECTION_ATTRIBUTE_CONTENTS       = (1 << 0),
        SECTION_ATTRIBUTE_MANDATORY      = (1 << 1),
        SECTION_ATTRIBUTE_ANCHORED       = (1 << 4),
        SECTION_ATTRIBUTE_INITIALIZATION = (1 << 5),
        SECTION_ATTRIBUTE_MASK           = SECTION_ATTRIBUTE_CONTENTS |
                                           SECTION_ATTRIBUTE_MANDATORY |
                                           SECTION_ATTRIBUTE_ANCHORED |
                                           SECTION_ATTRIBUTE_INITIALIZATION
} SectionAttribute;

typedef unsigned SectionAttributes;

typedef enum
{
        SECTION_CAPABILITY_READ    = (1 << 5),
        SECTION_CAPABILITY_WRITE   = (1 << 6),
        SECTION_CAPABILITY_EXECUTE = (1 << 7),
        SECTION_CAPABILITY_MASK    = SECTION_CAPABILITY_READ |
                                     SECTION_CAPABILITY_WRITE |
                                     SECTION_CAPABILITY_EXECUTE
} SectionCapability;

typedef unsigned SectionCapabilities;

typedef struct
{
        ListNode                sectionList;
        struct Object *         object;
        SectionType             type;
        SectionAttributes       attributes;
        SectionCapabilities     capabilities;
        const char *            name;
        const char *            architecture;
        Address                 address;
        size_t                  dataAlignment;
        size_t                  dataSize;
        uint8 *                 data;
        List                    relocations;
        List                    exportedSymbols;
} Section;

extern Section *       section_create   (struct Object *,
                                         SectionType,
                                         SectionAttributes,
                                         SectionCapabilities,
                                         const char *,
                                         const char *,
                                         Address,
                                         size_t,
                                         size_t,
                                         uint8 *);
extern void            section_destroy  (Section *);

extern int             section_adjacent (const Section *, const Section *);
extern int             section_merge    (Section *, Section *);
extern int             section_move     (Section *, struct Object *);
extern Address         section_anchor   (Section *, Address);
extern void            section_fixup    (Section *, Section *, size_t); 
extern void            section_relocate (Section *);

extern int             section_read     (Section *, FILE *);
extern int             section_write    (const Section *, FILE *, int);

#endif /* __XL_SECTION_H__ */

