#include <stdlib.h>
#include "error.h"
#include "object.h"
#include "symbol.h"

Symbol *
symbol_create (Section * section, 
               SymbolType type,
               SymbolAttributes attributes,
               Definition * definition,
               Address value,
               size_t size)
{
    Symbol * symbol = (Symbol *) malloc (sizeof (Symbol));

    if (symbol == NULL)
      return NULL;

    if (type != SYMBOL_TYPE_CONSTANT &&
        type != SYMBOL_TYPE_FILE &&
        (attributes & SYMBOL_ATTRIBUTE_IMPORTED) == 0 &&
        section == NULL)
      return NULL;

    symbol -> section         = section;
    symbol -> type            = type;
    symbol -> attributes      = attributes;
    symbol -> definition      = definition;
    symbol -> value           = value;
    symbol -> size            = size;

    if (section != NULL &&
        type != SYMBOL_TYPE_SECTION)
      list_insert_front (& section -> exportedSymbols, & symbol -> symbolList);

    return symbol;
}

void
symbol_destroy (Symbol * symbol)
{
    Definition * definition = symbol -> definition;

    if (definition -> symbol == symbol)
      definition -> symbol = NULL;

    list_remove (& symbol -> symbolList);

    free (symbol);

    definition_dereference (definition);
}

Address
symbol_value (const Symbol * symbol)
{
    if (symbol -> type == SYMBOL_TYPE_CONSTANT ||
        symbol -> type == SYMBOL_TYPE_FILE ||
        (symbol -> attributes & SYMBOL_ATTRIBUTE_IMPORTED) != 0)
      return symbol -> value;

    return symbol -> section -> address + symbol -> value;
}

int
symbol_resolved (const Symbol * symbol)
{
    if (symbol -> type == SYMBOL_TYPE_CONSTANT ||
        symbol -> type == SYMBOL_TYPE_FILE)
      return 1;

    if (symbol -> attributes & SYMBOL_ATTRIBUTE_IMPORTED)
      return 0;

    if (symbol -> section -> attributes & SECTION_ATTRIBUTE_ANCHORED)
      return 1;

    return 0;
}

