#ifndef __XL_SYMBOL_H__
#define __XL_SYMBOL_H__

#include <stdlib.h>
#include "types.h"
#include "list.h"
#include "section.h"
#include "definition.h"

struct Object;

typedef enum
{
        SYMBOL_TYPE_SECTION     = 0,
        SYMBOL_TYPE_CONSTANT    = 1,
        SYMBOL_TYPE_GENERIC     = 2,
        SYMBOL_TYPE_FUNCTION    = 3,
        SYMBOL_TYPE_DATA        = 4,
        SYMBOL_TYPE_FILE        = 5
} SymbolType;

typedef enum
{
        SYMBOL_ATTRIBUTE_EXPORTED  = (1 << 0),
        SYMBOL_ATTRIBUTE_IMPORTED  = (1 << 1),
        SYMBOL_ATTRIBUTE_MANDATORY = (1 << 2),
        SYMBOL_ATTRIBUTE_COMMON    = (1 << 3)
} SymbolAttribute;

typedef unsigned SymbolAttributes;

typedef struct Symbol
{
        ListNode                symbolList;
        Section *               section;
        SymbolType              type;
        SymbolAttributes        attributes;
        Definition *            definition;
        Address                 value;
        size_t                  size;
} Symbol;

extern Symbol *    symbol_create   (Section *,
                                    SymbolType, 
                                    SymbolAttributes, 
                                    Definition *,
                                    Address,
                                    size_t);
extern void        symbol_destroy  (Symbol *);

extern void        symbol_merge    (Section *, Section *);
extern void        symbol_move     (struct Object *, struct Object *);
extern Address     symbol_value    (const Symbol *);
extern int         symbol_resolved (const Symbol *);

#endif /* __XL_SYMBOL_H__ */

        
