#ifndef __XL_XCOM_H__
#define __XL_XCOM_H__

#include "types.h"

typedef uint8    	xc_uint8;
typedef uint16   	xc_uint16;
typedef uint32  	xc_uint32;

typedef xc_uint32	xc_reference;

static const char XCOM_SIGNATURE [6]				        = { 'X', 'C', 'O', 'M', (char) 0x19, (char) 0x98 };

static const char * const XCOM_HOST_ARCHITECTURE_CODE			= "ia32";
static const char * const XCOM_HOST_ARCHITECTURE_DATA			= "little-ia32";

static const xc_reference XCOM_HOST_ALIGNMENT_CODE                      = 4;
static const xc_reference XCOM_HOST_ALIGNMENT_DATA                      = 5;

static const xc_reference XCOM_HOST_REFERENCE_ATTRIBUTE_BIG_ENDIAN	= 0;
static const xc_reference XCOM_HOST_REFERENCE_ATTRIBUTE_SIZE		= 5;

typedef enum
{
	XCOM_SIGNATURE_LENGTH	= 6,
	
	XCOM_SIGNATURE_0	= 'X',
	XCOM_SIGNATURE_1	= 'C',
	XCOM_SIGNATURE_2	= 'O',
	XCOM_SIGNATURE_3	= 'M',
	XCOM_SIGNATURE_4	= 0x19,
	XCOM_SIGNATURE_5	= 0x98
} XComSignature;

typedef enum
{
	XCOM_VERSION	= 1
} XComVersion;

typedef enum
{
	XCOM_REFERENCE_ATTRIBUTE_SIZE		= 0x1F,
	XCOM_REFERENCE_ATTRIBUTE_ZERO_MASK	= 0x60,
	XCOM_REFERENCE_ATTRIBUTE_BIG_ENDIAN	= 0x80
} XComReferenceAttribute;

typedef enum
{
	XCOM_SECTION_TYPE_STRINGS		= 1,
	XCOM_SECTION_TYPE_INTERFACE		= 2,
	XCOM_SECTION_TYPE_RESOURCE		= 3,
	XCOM_SECTION_TYPE_PROGRAM		= 4,
	XCOM_SECTION_TYPE_SYMBOLS		= 5,
	XCOM_SECTION_TYPE_RELOCATIONS		= 6
} XComSectionType;

typedef enum
{
	XCOM_SECTION_ATTRIBUTE_CONTENTS		= (1 << 0),
	XCOM_SECTION_ATTRIBUTE_MANDATORY	= (1 << 1),
	XCOM_SECTION_ATTRIBUTE_COMPLETE		= (1 << 2),
	XCOM_SECTION_ATTRIBUTE_RELOCATIONS	= (1 << 3),
	XCOM_SECTION_ATTRIBUTE_VMA		= (1 << 4),
	XCOM_SECTION_ATTRIBUTE_INITIALIZATION	= (1 << 5),
	XCOM_SECTION_ATTRIBUTE_DEBUG		= (1 << 6)
} XComSectionAttribute;

typedef enum
{
	XCOM_MEMORY_ATTRIBUTE_READ		= (1 << 5),
	XCOM_MEMORY_ATTRIBUTE_WRITE		= (1 << 6),
	XCOM_MEMORY_ATTRIBUTE_EXECUTE		= (1 << 7)
} XComMemoryAttribute;

typedef enum
{
	XCOM_SYMBOL_TYPE_SECTION		= 0,
	XCOM_SYMBOL_TYPE_CONSTANT		= 1,
	XCOM_SYMBOL_TYPE_GENERIC		= 2,
	XCOM_SYMBOL_TYPE_FUNCTION		= 3,
	XCOM_SYMBOL_TYPE_DATA			= 4,
        XCOM_SYMBOL_TYPE_FILE                   = 5
} XComSymbolType;

typedef enum
{
	XCOM_SYMBOL_ATTRIBUTE_GLOBAL		= (1 << 0),
	XCOM_SYMBOL_ATTRIBUTE_EXTERNAL		= (1 << 1),
	XCOM_SYMBOL_ATTRIBUTE_MANDATORY		= (1 << 2),
	XCOM_SYMBOL_ATTRIBUTE_COMMON		= (1 << 3)
} XComSymbolAttribute;

typedef enum
{
	XCOM_RELOCATION_TYPE_IA32_SEGMENT	= 1,
	XCOM_RELOCATION_TYPE_IA32_ABSOLUTE_16	= 2,
	XCOM_RELOCATION_TYPE_IA32_ABSOLUTE_32	= 3,
	XCOM_RELOCATION_TYPE_IA32_RELATIVE_8	= 4,
	XCOM_RELOCATION_TYPE_IA32_RELATIVE_16	= 5,
	XCOM_RELOCATION_TYPE_IA32_RELATIVE_32	= 6
} XComRelocationTypeIA32;

#define XCOM_ALIGN(offset,word) 		((offset) % (word))

#define XCOM_HEADER_SIZE(referenceSize) 	((referenceSize) * 2 + 3)

#define XCOM_HEADER(XComHeader, xc_reference)				\
struct XComHeader							\
{									\
	xc_uint8	signature [6];					\
	xc_uint8	referenceAttributes;				\
	xc_uint8	version;					\
									\
	xc_reference	fileSize;					\
	xc_reference	sections;					\
}					


#define XCOM_SECTION_SIZE(referenceSize) 	((referenceSize) * 5 + 4 + XCOM_ALIGN (4, referenceSize))

#define XCOM_SECTION(XComSection, xc_reference)				\
struct XComSection							\
{									\
	xc_uint8	type;						\
	xc_uint8	alignment;					\
	xc_uint8	sectionAttributes;				\
	xc_uint8	memoryAttributes;				\
									\
	unsigned	: 8 * XCOM_ALIGN (4, sizeof (xc_reference));	\
									\
	xc_reference	memoryRegion;					\
									\
	xc_reference	nameIndex;					\
	xc_reference	dataIndex;					\
									\
	xc_reference	size;						\
	xc_reference	offset;						\
}								

#define XCOM_SYMBOL_SIZE(referenceSize) 	((referenceSize) * 3 + 4 + XCOM_ALIGN (4, referenceSize))

#define XCOM_SYMBOL(XComSymbol, xc_reference)				\
struct XComSymbol							\
{									\
	xc_uint16	type;						\
	xc_uint16	attributes;					\
									\
	unsigned	: 8 * XCOM_ALIGN (4, sizeof (xc_reference));	\
									\
	xc_reference	size;						\
	xc_reference	offset;						\
	xc_reference	nameIndex;					\
}

#define XCOM_RELOCATION_SIZE(referenceSize) 	((referenceSize) * 3 + 4 + XCOM_ALIGN (4, referenceSize))

#define XCOM_RELOCATION(XComRelocation, xc_reference)			\
struct XComRelocation							\
{									\
	xc_uint16	relocationType;					\
	xc_uint16	symbolType;					\
									\
	unsigned	: 8 * XCOM_ALIGN (4, sizeof (xc_reference));	\
									\
	xc_reference	size;						\
	xc_reference	offset;						\
	xc_reference	nameIndex;					\
}

typedef XCOM_HEADER	(XCom8Header,		xc_uint8)	XCom8Header;
typedef XCOM_SECTION	(XCom8Section,		xc_uint8)	XCom8Section;
typedef XCOM_SYMBOL	(XCom8Symbol,		xc_uint8)	XCom8Symbol;
typedef XCOM_RELOCATION	(XCom8Relocation,	xc_uint8)	XCom8Relocation;

typedef XCOM_HEADER	(XCom16Header,		xc_uint16)	XCom16Header;
typedef XCOM_SECTION	(XCom16Section,		xc_uint16)	XCom16Section;
typedef XCOM_SYMBOL	(XCom16Symbol,		xc_uint16)	XCom16Symbol;
typedef XCOM_RELOCATION	(XCom16Relocation,	xc_uint16)	XCom16Relocation;

typedef XCOM_HEADER 	(XCom32Header, 		xc_uint32)	XCom32Header;
typedef XCOM_SECTION 	(XCom32Section, 	xc_uint32)	XCom32Section;
typedef XCOM_SYMBOL 	(XCom32Symbol, 		xc_uint32)	XCom32Symbol;
typedef XCOM_RELOCATION	(XCom32Relocation, 	xc_uint32)	XCom32Relocation;

typedef XCOM_HEADER	(XComHeader,		xc_reference)	XComHeader;
typedef XCOM_SECTION	(XComSection,		xc_reference)	XComSection;
typedef XCOM_SYMBOL	(XComSymbol,		xc_reference)	XComSymbol;
typedef XCOM_RELOCATION	(XComRelocation,	xc_reference)	XComRelocation;

#endif /* __XL_XCOM_H__ */

