#include <stdlib.h>
#include <string.h>
#include "xcom.h"
#include "error.h"
#include "object.h"
#include "symbol.h"
#include "section.h"
#include "relocation.h"

int
xcom_write (Object * object, FILE * file, int zeroFill)
{
     XComHeader header;
     XComSection * sectionMap,
                 * currentSection,
                 * targetSection;
     xc_reference currentOffset;
     ListIterator position;
     
     memcpy (header.signature, XCOM_SIGNATURE, XCOM_SIGNATURE_LENGTH);

     header.referenceAttributes = XCOM_HOST_REFERENCE_ATTRIBUTE_SIZE |
                                  XCOM_HOST_REFERENCE_ATTRIBUTE_BIG_ENDIAN;
     header.version             = XCOM_VERSION;

     sectionMap = (XComSection *) calloc (3 * list_size (& object -> sections) + 2, sizeof (XComSection));
     if (sectionMap == NULL)
       return -1;

     sectionMap -> type              = XCOM_SECTION_TYPE_STRINGS;
     sectionMap -> sectionAttributes = XCOM_SECTION_ATTRIBUTE_MANDATORY;
     sectionMap -> size              = namespace_assign_offsets (& object -> namespace);

     for (position = list_begin (& object -> sections),
             currentSection = & sectionMap [1];
          position != list_end (& object -> sections);
          position = list_next (position),
             ++ currentSection)
     {
         Section * section = (Section *) position;
       
         currentSection -> type              = section -> type;
         currentSection -> alignment         = section -> dataAlignment;
         currentSection -> sectionAttributes = section -> attributes;
         currentSection -> memoryAttributes  = section -> capabilities;
         currentSection -> memoryRegion      = (xc_reference) section -> address;
         currentSection -> size              = (xc_reference) section -> dataSize;

         if (section -> name != NULL)
           currentSection -> nameIndex = (xc_reference) namespace_find (& object -> namespace, section -> name) -> offset;

         if (section -> architecture != NULL)
           currentSection -> dataIndex = (xc_reference) namespace_find (& object -> namespace, section -> architecture) -> offset;

         if (zeroFill != 0 &&
             currentSection -> type == XCOM_SECTION_TYPE_PROGRAM)
           currentSection -> sectionAttributes |= XCOM_SECTION_ATTRIBUTE_CONTENTS;
     } 

     if (list_empty (& object -> importedSymbols) == 0 ||
         list_empty (& object -> constantSymbols) == 0)
     {
          currentSection -> type = XCOM_SECTION_TYPE_SYMBOLS;
          currentSection -> size = (list_size (& object -> importedSymbols) +
                                    list_size (& object -> constantSymbols)) * sizeof (XComSymbol);

          ++ currentSection;
     }

     for (position = list_begin (& object -> sections),
             targetSection = & sectionMap [1];
          position != list_end (& object -> sections);
          position = list_next (position),
             ++ targetSection)
     {
         Section * section = (Section *) position;

         if (list_empty (& section -> exportedSymbols) == 0)
         {
            currentSection -> type       = XCOM_SECTION_TYPE_SYMBOLS;
            currentSection -> dataIndex  = targetSection - sectionMap + 1;
            currentSection -> size       = list_size (& section -> exportedSymbols) * sizeof (XComSymbol);
            
            ++ currentSection;
         }

         if (list_empty (& section -> relocations) == 0)
         {
            ListIterator relocPos;

            for (relocPos = list_begin (& section -> relocations);
                 relocPos != list_end (& section -> relocations);
                 relocPos = list_next (relocPos))
               if (((Relocation *) relocPos) -> definition -> symbol -> type != SYMBOL_TYPE_SECTION)
                 break;
            
            targetSection -> sectionAttributes |= XCOM_SECTION_ATTRIBUTE_RELOCATIONS;
            if (relocPos == list_end (& section -> relocations))
              targetSection -> sectionAttributes |= XCOM_SECTION_ATTRIBUTE_COMPLETE;

            currentSection -> type               = XCOM_SECTION_TYPE_RELOCATIONS;
            currentSection -> sectionAttributes  = XCOM_SECTION_ATTRIBUTE_MANDATORY;
            currentSection -> dataIndex          = targetSection - sectionMap + 1;
            currentSection -> size               = list_size (& section -> relocations) * sizeof (XComRelocation);
            
            ++ currentSection;
         }
         else
         {
            targetSection -> sectionAttributes |= XCOM_SECTION_ATTRIBUTE_COMPLETE;
            targetSection -> sectionAttributes &= ~ XCOM_SECTION_ATTRIBUTE_RELOCATIONS;
         }
     }

     header.sections = currentSection - sectionMap;

     sectionMap -> offset = sizeof (XComHeader) + header.sections * sizeof (XComSection);
     
     currentOffset = sectionMap -> offset + sectionMap -> size;

     for (position = list_begin (& object -> sections),
             currentSection = & sectionMap [1];
          position != list_end (& object -> sections);
          position = list_next (position),
             ++ currentSection)
     { 
        Section * section = (Section *) position;
        xc_reference alignment, 
                     alignOffset;

        if (section -> type == XCOM_SECTION_TYPE_PROGRAM)
        { 
           if (zeroFill == 0 &&
               (section -> attributes & XCOM_SECTION_ATTRIBUTE_CONTENTS) == 0)
             continue;
        
           alignment   = (xc_reference) (1 << currentSection -> alignment);
           alignOffset = currentOffset % alignment;
           if (alignOffset > 0)
             currentOffset += alignment - alignOffset;
        }
        
        currentSection -> offset = currentOffset;

        currentOffset += currentSection -> size;
     }
    
     while (currentSection < & sectionMap [header.sections])
     {
        currentSection -> offset = currentOffset;

        currentOffset += currentSection -> size;

        ++ currentSection; 
     }

     header.fileSize = currentOffset;
    
     if (fwrite (& header, sizeof (XComHeader), 1, file) != 1)
       xl_error ("Could not write XCOM header");

     if (fwrite (sectionMap, header.sections * sizeof (XComSection), 1, file) != 1)
       xl_error ("Could not write section map for XCOM");

     if (namespace_write (& object -> namespace, file) != 0)
       xl_error ("Could not write string section for XCOM");

     for (position = list_begin (& object -> sections),
             currentSection = & sectionMap [1];
          position != list_end (& object -> sections);
          position = list_next (position),
             ++ currentSection)
     {        
         Section * section = (Section *) position;

         if (currentSection -> offset <= 0)
           continue;

         if (fseek (file, currentSection -> offset, SEEK_SET) != 0 ||
             section_write (section, file, zeroFill) != 0)
           xl_error ("Could not write data for section %s for XCOM", section -> name);
     }

     if (list_empty (& object -> importedSymbols) == 0)
     {
         ListIterator symbolPos;

         for (symbolPos = list_begin (& object -> importedSymbols);
              symbolPos != list_end (& object -> importedSymbols);
              symbolPos = list_next (symbolPos))
         {
             Symbol * symbol = (Symbol *) symbolPos;
             XComSymbol currentSymbol;

             currentSymbol.type       = symbol -> type;
             currentSymbol.attributes = symbol -> attributes;
             currentSymbol.size       = 0;
             currentSymbol.offset     = (xc_reference) symbol -> value;
             currentSymbol.nameIndex  = (xc_reference) symbol -> definition -> offset;

             if (fwrite (& currentSymbol, sizeof (XComSymbol), 1, file) != 1)
               xl_error ("Could not write imported symbol %s for XCOM", symbol -> definition -> name);
         }
     }

     if (list_empty (& object -> constantSymbols) == 0)
     {
         ListIterator symbolPos;

         for (symbolPos = list_begin (& object -> constantSymbols);
              symbolPos != list_end (& object -> constantSymbols);
              symbolPos = list_next (symbolPos))
         {
             Symbol * symbol = (Symbol *) symbolPos;
             XComSymbol currentSymbol;

             currentSymbol.type       = symbol -> type;
             currentSymbol.attributes = symbol -> attributes;
             currentSymbol.size       = 0;
             currentSymbol.offset     = (xc_reference) symbol -> value;
             currentSymbol.nameIndex  = (xc_reference) symbol -> definition -> offset;

             if (fwrite (& currentSymbol, sizeof (XComSymbol), 1, file) != 1)
               xl_error ("Could not write constant symbol %s for XCOM", symbol -> definition -> name);
         }
     }

     for (position = list_begin (& object -> sections);
          position != list_end (& object -> sections);
          position = list_next (position))
     {
         Section * section = (Section *) position;
         ListIterator symbolPos, relocPos;

         for (symbolPos = list_begin (& section -> exportedSymbols);
              symbolPos != list_end (& section -> exportedSymbols);
              symbolPos = list_next (symbolPos))
         {
             Symbol * symbol = (Symbol *) symbolPos;
             XComSymbol currentSymbol;

             currentSymbol.type       = symbol -> type;
             currentSymbol.attributes = symbol -> attributes;
             currentSymbol.size       = 0;
             currentSymbol.offset     = (xc_reference) symbol -> value;
             currentSymbol.nameIndex  = (xc_reference) symbol -> definition -> offset;

             if (fwrite (& currentSymbol, sizeof (XComSymbol), 1, file) != 1)
               xl_error ("Could not write exported symbol %s for XCOM", symbol -> definition -> name);
         }

         for (relocPos = list_begin (& section -> relocations);
              relocPos != list_end (& section -> relocations);
              relocPos = list_next (relocPos))
         {
             Relocation * reloc = (Relocation *) relocPos;
             XComRelocation currentReloc;

             currentReloc.relocationType = reloc -> type;
             currentReloc.symbolType     = reloc -> definition -> symbol -> type;
             currentReloc.size           = 0;
             currentReloc.offset         = (xc_reference) reloc -> offset;
             currentReloc.nameIndex      = (xc_reference) reloc -> definition -> offset;

             if (fwrite (& currentReloc, sizeof (XComRelocation), 1, file) != 1)
               xl_error ("Could not write relocation against %s in section %s for XCOM", 
                         reloc -> definition -> name,
                         section -> name);
         }        
     }

     free (sectionMap);

     return 0;
}
                 
